#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Name:     iso.py
# Purpose:  Module to manupulate ISO file
# Authors:  Sundar
# Depends:  isodump3.py (Authored by Johni Lee for MultiBootUSB)
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import re
from .gen import *
from .isodump3 import ISO9660
from . import _7zip


_iso_cfg_ext_dir = iso_cfg_ext_dir()


def iso_name(iso_link):
    """
    Find the name of an ISO.
    :return: Name of an ISO (with extension) as string. Returns If not returns None.
    """
    if os.path.exists(iso_link):
        try:
            name = os.path.basename(str(iso_link))
        except:
            name = None
    else:
        name = None

    return name


def iso_basename(iso_link):
    """
    Find the base name of an ISO.
    :return: Base name (without extension) of a selected ISO as string. If not returns None.
    """
    try:
        dir_name = str(os.path.splitext(os.path.basename(str(iso_link)))[0])
    except:
        dir_name = None

    return dir_name


def isolinux_bin_exist(iso_link):
    """
    Check if an "isolinux.bin" file exist.
    :return: True if "isolinux.bin" file exist of False if not.
    """
    if os.path.exists(iso_link):
        iso_file_list = _7zip.list_iso(iso_link)
        return bool(any("isolinux.bin" in s.lower() for s in iso_file_list))


def iso_size(iso_link):
    return os.path.getsize(iso_link)


def is_readable(iso_link):
    return os.access(iso_link, os.R_OK)


def is_bootable(iso_link):
    """
    Check if an ISO has the ability to boot.
    :return: True if ISO is bootable and False if not.
    """
    try:
        iso9660fs = ISO9660(iso_link)
    except IOError as e:
        log(str(e))
        raise
    isBootable = iso9660fs.checkISOBootable()
    return bool(isBootable)


def isolinux_bin_dir(iso_link):
    """
    Detects "isolinux.bin" directory.
    :return: path of "isolinux.bin" directory as string.
    """
    if os.path.exists(iso_link):
        bin_dir = False
        iso_file_list = _7zip.list_iso(iso_link)
        if any("isolinux.bin" in s.lower() for s in iso_file_list):
            for f in iso_file_list:
                if 'isolinux.bin' in f.lower():
                    if 'efi' not in f.lower():  # Certain distros place their isolinux.bin in to /EFI/BOOT director and we don't want to include them
                        bin_dir = os.path.dirname(f)
                        break

        return bin_dir


def isolinux_bin_path(iso_link):
    """
    Detects pat to "isolinux.bin".
    :return: path of "isolinux.bin" as a string.
    """
    iso_bin_path = False
    if isolinux_bin_exist(iso_link) is not False:
        iso_file_list = _7zip.list_iso(iso_link)
        for f in iso_file_list:
            if 'isolinux.bin' in f.lower():
                iso_bin_path = f
                break

    return iso_bin_path


def iso_menu_lst_path(iso_link):
    """
    Detects pat to "menu.lst" of grub4dos.
    :return: path of "menu.lst" as a string.
    """
    menu_lst_path = False
    iso_file_list = _7zip.list_iso(iso_link)
    for f in iso_file_list:
        if 'menu.lst' in f.lower():
            menu_lst_path = f
            break

    return menu_lst_path


def integrity(iso_link):
    """
    Check the integrity of an ISO.
    :return: True if integrity passes or False if it fails.
    """
    return _7zip.test_iso(iso_link)


def iso_file_list(iso_link):
    """
    Function to return the content of an ISO.
    :return: List of files of an ISO as list.
    """
    return _7zip.list_iso(iso_link)


def isolinux_version(isolinux_bin_path):
    """
    Detect isolinux version shipped by distros.
    :param isolinux_path: Path to "isolinux.bin"
    :return: Version number as string.
    """
    version = ["2", "3", "4", "5", "6"]
    if isolinux_bin_path is not None:
        sl = list(strings(isolinux_bin_path))
        for strin in sl:
            if re.search(r'isolinux ', strin, re.I):
                for number in version:
                    if re.search(r'isolinux ' + number, strin, re.I):
                        log("\nFound syslinux version " + number + "\n")
                        return str(number)


def iso_extract_file(iso_link, dest_dir, _filter):
    """
    Extract the specific file(s) from an ISO
    :param dest_dir: Path to destination directory.
    :param filter: Filter to extract particular file(s)
    :return: Extract file(s) to destination.
    """
    _7zip.extract_iso(iso_link, dest_dir, _filter)


def extract_cfg_file(iso_link):
    """
    Function to extract certain files for auto detecting supported distros
    :param iso_link: Path to ISO file
    :return:
    """
    #_pattern = ['.cfg', '.CFG', '.txt', '.TXT', 'isolinux.bin', 'ISOLINUX.BIN', '.lst']
    _pattern = ['.cfg', '.txt', 'isolinux.bin', '.lst']
    # file_list = iso_file_list(iso_link)
    for ext in _pattern:
        _7zip.extract_iso(iso_link, _iso_cfg_ext_dir, pattern='*' + ext)


def iso_extract_full(iso_link, dest_dir):
    """
    Extract an ISO to destination directory
    :param dest_dir: Destination path as string.
    :return: False if it fails or extract ISO files to destination directory.
    """
    _7zip.extract_iso(iso_link, dest_dir)


def iso_file_path(iso_link, file_name):
    """
    Function to check if file name exist in the ISO
    :param iso_link:Path to ISO file
    :param file_name: file name to be checked
    :return: Path to file as string
    """
    assert iso_link
    assert file_name

    file_path = False
    iso_file_list = _7zip.list_iso(iso_link)
    for f in iso_file_list:
        if file_name in f.lower():
            file_path = f
            break

    return file_path

if __name__ == '__main__':
    #iso_path = '../../../DISTROS/2016/debian-live-8.3.0-amd64-lxde-desktop.iso'
    iso_path = '../../../DISTROS/2015/super_grub2_disk_hybrid_2.02s3.iso'
    test_iso_bin_path = os.path.join('test', 'isolinux', 'isolinux.bin')
    log('iso_name(iso_path) : ', iso_name(iso_path))
    log('iso_basename(iso_path) : ', iso_basename(iso_path))
    log('Integrity of ISO is : ', integrity(iso_path))
    f_list = (iso_file_list(iso_path))
    if f_list:
        for f in f_list:
            log(f)
    log('isolinux_bin_exist(iso_path) : ', isolinux_bin_exist(iso_path))
    #log('is_bootable : ', is_bootable(iso_path))
    log('isolinux_bin_dir() : ', isolinux_bin_dir(iso_path))
    log('isolinux_bin_path(iso_path) : ', isolinux_bin_path(iso_path))
    iso_extract_full(iso_path, 'test')
    iso_extract_file(iso_path, 'test', 'isolinux.bin')
    log(isolinux_version(test_iso_bin_path))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     mbusb_gui.py
# Purpose:  Module to handle multibootusb through gui
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import platform
import sys
import signal
from PyQt5 import QtCore, QtGui, QtWidgets
import subprocess
import time
import webbrowser
from scripts.gui.ui_multibootusb import Ui_MainWindow
from scripts.gui.ui_about import Ui_About
from . import usb
from .gen import *
from .install import *
from .uninstall_distro import *
from .syslinux import *
from .distro import *
from .qemu import *
from .iso import *
# from .imager import *
from .imager import Imager, dd_linux, dd_win
from . import persistence
from . import config
from . import admin
from . import qemu
from .update_cfg_file import update_distro_cfg_files
import scripts.gui.resources


class AppGui(qemu.Qemu, Imager, QtWidgets.QMainWindow, Ui_MainWindow):
    """
    Main multibootusb GUI manipulation class.
    """

    def __init__(self):
        QtWidgets.QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.ui.tabWidget.setCurrentIndex(0)
        #       self.qemu = Qemu()

        self.ui.label_persistence_value.setVisible(False)
        self.ui.label_persistence.setVisible(False)
        self.ui.slider_persistence.setVisible(False)

        config.usb_disk = None
        config.image_path = None

        #  Main Tab
        self.ui.checkbox_all_drives.clicked.connect(self.onAllDrivesClicked)
        self.ui.button_detect_drives.clicked.connect(self.onRefreshClick)
        self.ui.action_Quit.triggered.connect(self.on_close_Click)
        self.ui.action_About.triggered.connect(self.onAboutClick)
        self.ui.button_browse_image.clicked.connect(self.browse_iso)
        #         self.ui.combo_drives.activated[str].connect(self.onComboChange)
        self.ui.combo_drives.currentIndexChanged.connect(self.onComboChange)
        self.ui.button_install_distro.clicked.connect(self.onCreateClick)
        self.ui.button_uninstall_distro.clicked.connect(self.OnUninstallClick)
        self.ui.slider_persistence.valueChanged.connect(self.update_slider_text)
        #         self.ui.slider_persistence.sliderReleased.connect(self.ui_update_persistence)

        # ISO Imager Tab
        self.ui.button_write_image_to_disk.clicked.connect(self.dd_write)

        #  Syslinux Tab
        self.ui.button_install_syslinux.clicked.connect(self.onInstall_syslinuxClick)
        self.ui.button_edit_syslinux.clicked.connect(self.onedit_syslinux)

        # QEMU Tab
        self.ui.boot_iso_qemu.clicked.connect(self.on_Qemu_Boot_iso_Click)
        self.ui.boot_usb_qemu.clicked.connect(self.on_Qemu_Boot_usb_Click)
        #         self.ui.combo_iso_boot_ram.activated[str].connect(self.qemu_iso_ram)
        #         self.ui.combo_usb_boot_ram.activated[str].connect(self.qemu_usb_ram)
        #         self.ui.boot_usb_qemu.clicked.connect(lambda: self.on_Qemu_Boot_usb_Click(str(self.ui.combo_drives.currentText())))
        #  Update progressbar and status  (Main ISO install)
        self.progress_thread_install = GuiInstallProgress()
        self.progress_thread_install.finished.connect(self.install_syslinux)
        self.progress_thread_install.update.connect(self.ui.progressbar.setValue)
        self.progress_thread_install.status.connect(self.ui.statusbar.showMessage)

        #  Update progressbar and status  (Uninstall from previous install)
        self.progress_thread_uninstall = GuiUninstallProgress()
        self.progress_thread_uninstall.finished.connect(self.uninstall_sys_file_update)
        self.progress_thread_uninstall.update.connect(self.ui.progressbar.setValue)
        self.progress_thread_uninstall.status.connect(self.ui.statusbar.showMessage)

        #  Update progressbar and status  (dd ISO)
        self.progress_thread_dd = DD_Progress()
        self.progress_thread_dd.update.connect(self.ui.progressbar.setValue)
        self.progress_thread_dd.finished.connect(self.dd_finished)
        self.progress_thread_dd.status.connect(self.ui.statusbar.showMessage)

        prepare_mbusb_host_dir()
        self.onRefreshClick()

    def onAllDrivesClicked(self):
        """
        Include fixed drives to available USB devices.
        :return:
        """
        if self.ui.checkbox_all_drives.isChecked() is False:
            self.onRefreshClick()
            return

        reply = QtWidgets.QMessageBox.warning(self, "WARNING!",
                                              "This option enables working with fixed drives\n\
and is potentially VERY DANGEROUS\n\n\
Are you SURE you want to enable it?",
                                              QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                              QtWidgets.QMessageBox.No)

        if reply == QtWidgets.QMessageBox.No:
            self.ui.checkbox_all_drives.setChecked(False)
        elif reply == QtWidgets.QMessageBox.Yes:
            self.ui.checkbox_all_drives.setChecked(True)
            self.onRefreshClick()

    def onAboutClick(self):
        about = QtWidgets.QDialog()
        about.ui = Ui_About()
        about.ui.setupUi(about)
        about.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        about.setWindowTitle("About MultiBootUSB - " + mbusb_version())
        about.setWindowIcon(QtGui.QIcon(resource_path(os.path.join("data", "tools", "multibootusb.png"))))
        about.ui.button_close.clicked.connect(about.close)
        about.ui.label_6.linkActivated.connect(webbrowser.open_new_tab)
        about.exec_()

    def onComboChange(self):
        """
        Detects and updates GUI with populated USB device details.
        :return:
        """
        self.ui.installed_distros.clear()
        config.usb_disk = str(self.ui.combo_drives.currentText())
        config.imager_usb_disk = str(self.ui.combo_drives.currentText())

        if config.usb_disk:
            log("Selected device " + config.usb_disk)

            config.usb_details = usb.details(config.usb_disk)
            config.persistence_max_size = persistence.max_disk_persistence(config.usb_disk)
            config.usb_mount = config.usb_details.get('mount_point', "")
            self.ui.usb_dev.setText(config.usb_disk)

            self.ui.usb_vendor.setText(config.usb_details.get('vendor', ""))
            self.ui.usb_model.setText(config.usb_details.get('model', ""))
            self.ui.usb_size.setText(str(usb.bytes2human(config.usb_details.get('size_total', ""))))
            self.ui.usb_mount.setText(config.usb_details.get('mount_point', ""))
            self.ui.usb_type.setText(config.usb_details.get('devtype', ""))
            self.ui.usb_fs.setText(config.usb_details.get('file_system', ""))

            # Get the GPT status of the disk and store it on a variable
            usb.gpt_device(config.usb_disk)

            self.update_list_box(config.usb_disk)
            self.ui_update_persistence()
        else:
            self.ui.usb_dev.clear()
            self.ui.usb_vendor.clear()
            self.ui.usb_model.clear()
            self.ui.usb_size.clear()
            self.ui.usb_mount.clear()
            self.ui.usb_type.clear()
            self.ui.usb_fs.clear()

            log("No USB disk found...")

    def onRefreshClick(self):
        """
        Calls function to detect USB devices.
        :return:
        """
        self.ui.combo_drives.clear()
        if self.ui.checkbox_all_drives.isChecked():
            detected_devices = usb.list_devices(fixed=True)
        else:
            detected_devices = usb.list_devices()

        if detected_devices:
            for device in detected_devices:
                self.ui.combo_drives.addItem(str(device))
            self.ui.combo_drives.setCurrentIndex(0)

    def update_list_box(self, usb_disk):
        """
        Updates listbox with installed distros on selected USB disk.
        :param usb_mount: Selected USB disk from combobox.
        :return:
        """
        distro_list = install_distro_list()
        if distro_list is not None:
            self.ui.installed_distros.clear()
            for name in distro_list:
                self.ui.installed_distros.addItem(name)
        else:
            if not config.usb_mount:
                log("USB disk is not mounted and can't update list widget...")

    def browse_iso(self):
        if str(self.ui.image_path.text()):
            self.ui.image_path.clear()
        preference_file_path = os.path.join(multibootusb_host_dir(), "preference", "iso_dir.txt")
        dir_path = ''
        if os.path.exists(preference_file_path):
            dir_path = open(preference_file_path, 'r').read()

        config.image_path = QtWidgets.QFileDialog.getOpenFileName(self, 'Select an iso...', dir_path,
                                                                  'ISO Files (*.iso);; Zip Files(*.zip);; '
                                                                  'Img Files(*.img);; All Files(*.*)')[0]

        if config.image_path:
            # sanity checks
            if not is_readable(config.image_path):
                QtWidgets.QMessageBox.critical(
                    self,
                    "ISO Not readable",
                    "Sorry, the file \"{0}\" is not readable.".format(config.image_path)
                )
                return
            if iso_size(config.image_path) == 0:
                QtWidgets.QMessageBox.critical(
                    self,
                    "ISO is an empty file",
                    "Sorry, the file \"{0}\" contains no data.".format(config.image_path)
                )
                return

            default_dir_path = os.path.dirname(config.image_path)
            gen.write_to_file(preference_file_path, default_dir_path)

            if platform.system() == "Windows":
                if "/" in config.image_path:
                    config.image_path = config.image_path.strip().replace("/", "\\")

            self.ui.image_path.insert(str(config.image_path))
            self.ui.label_image_size_value.setText(str(bytes2human(iso_size(config.image_path))))
            self.ui.label_image_size_value.setVisible(True)
            self.ui.label_image_bootable_value.setText(str(is_bootable(config.image_path)))
            self.ui.label_image_bootable_value.setVisible(True)

            if os.path.exists(config.image_path):
                clean_iso_cfg_ext_dir(os.path.join(multibootusb_host_dir(), "iso_cfg_ext_dir"))  # Need to be cleaned.
                extract_cfg_file(config.image_path)
                config.distro = distro(iso_cfg_ext_dir(), config.image_path)  # Detect supported distro
                self.ui.label_image_type_value.setText(str(config.distro))
                self.ui.label_image_type_value.setVisible(True)
                if config.distro:
                    per_availability = persistence.persistence_distro(config.distro, config.image_path)
                    if per_availability is not None:
                        config.persistence_available = True
                        if config.usb_disk:
                            per_max_size = persistence.max_disk_persistence(config.usb_disk)
                            config.persistence_max_size = per_max_size
                            log('Persistence Max Size: ' + str(bytes2human(per_max_size)))
                    else:
                        config.persistence_available = False
                        log('Persistence is not available for ' + iso_name(config.image_path))

                    self.ui_update_persistence()
        else:
            log("File not selected...")

    def ui_update_persistence(self):
        #         log("===== config.persistence_available = " + str(config.persistence_available))
        #         log("===== config.persistence_max_size = " + str(config.persistence_max_size))
        #         log("===== config.persistence = " + str(config.persistence))
        if config.persistence_available and config.persistence_max_size:
            self.ui.label_persistence_value.setVisible(True)
            self.ui.label_persistence.setVisible(True)
            self.ui.slider_persistence.setVisible(True)
            self.ui.label_persistence_value.setEnabled(True)
            self.ui.label_persistence.setEnabled(True)
            self.ui.slider_persistence.setEnabled(True)
            self.ui.slider_persistence.setTickInterval(10)
            self.ui.slider_persistence.setSingleStep(10)
            self.ui.slider_persistence.setMaximum(config.persistence_max_size / 1024 / 1024)
        #             log("===== getMaximum = " + self.ui.slider_persistence.getMaximum()
        else:
            self.ui.label_persistence_value.setEnabled(False)
            self.ui.label_persistence.setEnabled(False)
            self.ui.slider_persistence.setEnabled(False)
            self.ui.label_persistence_value.setVisible(False)
            self.ui.label_persistence.setVisible(False)
            self.ui.slider_persistence.setVisible(False)

    def ui_disable_persistence(self):
        self.ui.label_persistence_value.setEnabled(False)
        self.ui.label_persistence.setEnabled(False)
        self.ui.slider_persistence.setEnabled(False)
        self.ui.label_persistence_value.setVisible(False)
        self.ui.label_persistence.setVisible(False)
        self.ui.slider_persistence.setVisible(False)

    def ui_disable_controls(self):
        self.ui.combo_drives.setEnabled(False)
        self.ui.checkbox_all_drives.setEnabled(False)
        self.ui.button_detect_drives.setEnabled(False)
        self.ui.button_browse_image.setEnabled(False)
        self.ui.image_path.setEnabled(False)
        self.ui.tabWidget.setEnabled(False)

    def ui_enable_controls(self):
        self.ui.combo_drives.setEnabled(True)
        self.ui.checkbox_all_drives.setEnabled(True)
        self.ui.button_detect_drives.setEnabled(True)
        self.ui.button_browse_image.setEnabled(True)
        self.ui.image_path.setEnabled(True)
        self.ui.tabWidget.setEnabled(True)

    def update_slider_text(self):
        slide_value = self.ui.slider_persistence.value() * 1024 * 1024
        self.ui.label_persistence_value.setText(bytes2human(slide_value))
        config.persistence = slide_value

    def install_syslinux(self):
        """
        Function to install syslinux on distro directory and on selected USB disks.
        :return:
        """
        self.ui.statusbar.showMessage(str("Status: Installing Syslinux..."))
        syslinux_distro_dir(config.usb_disk, config.image_path, config.distro)
        syslinux_default(config.usb_disk)
        replace_grub_binary()
        update_distro_cfg_files(config.image_path, config.usb_disk, config.distro, config.persistence)
        self.update_list_box(config.usb_disk)
        if sys.platform.startswith("linux"):
            self.ui.statusbar.showMessage("Status: Sync is in progress...")
            os.sync()
        self.ui.statusbar.showMessage("Status: Idle")
        self.ui_disable_persistence()
        log(iso_name(config.image_path) + ' has been successfully installed.')
        QtWidgets.QMessageBox.information(self, 'Finished...',
                                          iso_name(config.image_path) + ' has been successfully installed.')
        config.process_exist = None
        self.ui_enable_controls()

    def onInstall_syslinuxClick(self):
        """
        Function to install syslinux/extlinux on selected USB disk.
        :return:
        """

        self.ui_disable_controls()
        if platform.system() == "Linux" or platform.system() == "Windows":
            if self.ui.check_install_sys_all.isChecked() or self.ui.check_install_sys_only.isChecked():
                if platform.system() == 'Linux' and config.usb_disk[-1].isdigit() is False:
                    gen.log('Selected USB is a disk. Please select a disk partition from the drop down list')
                    QtWidgets.QMessageBox.information(self, 'No Partition...!',
                                                      'USB disk selected doesn\'t contain a partition.\n'
                                                      'Please select the partition (ending '
                                                      'with a digit eg. /dev/sdb1)\nfrom the drop down list.')

                else:
                    log("Installing default syslinux on " + config.usb_disk)
                    ret = syslinux_default(config.usb_disk)
                    if ret is True:
                        if self.ui.check_install_sys_all.isChecked():
                            log("Copying multibootusb directory to " + config.usb_mount)
                            for dirpath, dirnames, filenames in os.walk(
                                    resource_path(os.path.join("tools", "multibootusb"))):
                                for f in filenames:
                                    log("Copying " + f)
                                    shutil.copy(resource_path(os.path.join(dirpath, f)),
                                                os.path.join(self.usb.get_usb(config.usb_disk).mount, "multibootusb"))
                        QtWidgets.QMessageBox.information(self, 'Install Success...',
                                                          'Syslinux installed successfully on ' + config.usb_disk)
                    elif ret is False:
                        QtWidgets.QMessageBox.information(self, 'Install error...',
                                                          'Sorry. Syslinux failed to install on ' + config.usb_disk)
            else:
                QtWidgets.QMessageBox.information(self, 'No selection...',
                                                  'Please select one of the option from above.')

        self.ui_enable_controls()

    def onedit_syslinux(self):
        """
        Function to edit main syslinux.cfg file.
        :return:
        """
        # Function to edit syslinux.cfg file on editors like gedit, notepad etc.
        # Suggest me more editor which can be included in to this function.
        sys_cfg_file = os.path.join(config.usb_mount, "multibootusb", "syslinux.cfg")
        log("Locating " + sys_cfg_file)
        editor = ''
        if not os.path.exists(sys_cfg_file):
            log("syslinux.cfg file not found...")
            QtWidgets.QMessageBox.information(self, 'File not found...', 'Sorry. Unable to locate syslinux.cfg file.\n'
                                                                         'You can only edit syslinux.cfg file generated by multibootusb.')
        else:
            if platform.system() == "Linux":
                for e in config.editors_linux:
                    if subprocess.call('which ' + e, shell=True) == 0:
                        log("Editor found is " + e)
                        editor = e
                        break
            elif platform.system() == "Windows":
                for e in config.editors_win:
                    if not shutil.which(e) is None:
                        log("Editor found is " + e)
                        editor = e
                        break
            if not editor:
                QtWidgets.QMessageBox.information(self, 'Editor not found...',
                                                  'Sorry. Installed editor is not supported by multibootusb\n'
                                                  'Edit ' + sys_cfg_file + ' manually.\n')
            else:
                try:
                    subprocess.Popen(editor + " '" + sys_cfg_file + "'", shell=True).pid
                except OSError:
                    QtWidgets.QMessageBox.warning(self, 'Error...',
                                                  'Failed to open syslinux.cfg file.\n'
                                                  'Edit syslinux.cfg file manually.\n')

    def OnUninstallClick(self):
        """
        Triggers a function to uninstall a selected distro.
        :return:
        """

        self.ui_disable_controls()

        if self.ui.installed_distros.currentItem() is None:
            log("Please select a distro from the list.")
            QtWidgets.QMessageBox.information(self, 'No selection.', 'Please select a distro from the list.')
            self.ui_enable_controls()
        else:
            config.uninstall_distro_dir_name = str(self.ui.installed_distros.currentItem().text()).strip()
            reply = QtWidgets.QMessageBox.question(self, "Review selection...",
                                                   "Are you sure to uninstall " + config.uninstall_distro_dir_name,
                                                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                   QtWidgets.QMessageBox.No)

            if reply == QtWidgets.QMessageBox.Yes:
                if not os.path.exists(os.path.join(config.usb_mount, 'multibootusb', config.uninstall_distro_dir_name)):
                    log("Distro install directory not found. Just updating syslinux.cfg file.")
                    update_sys_cfg_file()
                    # self.uninstall.update_sys_cfg_file()
                    self.ui_enable_controls()
                else:
                    self.progress_thread_uninstall.start()
            else:
                self.ui_enable_controls()

    def uninstall_sys_file_update(self):
        """
        Function to remove and update uninstall distro text.
        :return:
        """
        update_sys_cfg_file()
        self.update_list_box(config.usb_mount)
        if sys.platform.startswith("linux"):
            self.ui.statusbar.showMessage("Status: Sync in progress...")
            os.sync()
        self.ui.statusbar.showMessage("Status: Idle")
        QtWidgets.QMessageBox.information(self, 'Uninstall Complete...',
                                          config.uninstall_distro_dir_name + ' has been successfully removed.')
        self.ui_enable_controls()

    def onCreateClick(self):
        """
        Main function to create bootable USB disk.
        :param usb_disk: ComboBox text as detected USB disk.
        :param iso_link: LineEdit text as selected ISO link.
        :return:
        """

        self.ui_disable_controls()

        if not config.usb_disk:
            log("ERROR: No USB device found.")
            QtWidgets.QMessageBox.information(self, "No Device...",
                                              "No USB device found.\n\nInsert USB and use Refresh USB button to detect USB.")
            self.ui_enable_controls()
        elif not config.image_path:
            log("No ISO selected.")
            QtWidgets.QMessageBox.information(self, "No ISO...", "No ISO found.\n\nPlease select an ISO.")
            self.ui_enable_controls()
        elif usb.details(config.usb_disk)['mount_point'] == 'No_Mount':
            log("ERROR: USB disk is not mounted.")
            QtWidgets.QMessageBox.information(self, "No Mount...", "USB disk is not mounted.\n"
                                                                   "Please mount USB disk and press refresh USB button.")
            self.ui_enable_controls()
        elif platform.system() == 'Linux' and config.usb_disk[-1].isdigit() is False:
            gen.log('Selected USB is a disk. Please select a disk partition from the drop down list')
            QtWidgets.QMessageBox.information(self, 'No Partition...!',
                                              'USB disk selected doesn\'t contain a partition.\n'
                                              'Please select the partition (ending '
                                              'with a digit eg. /dev/sdb1)\nfrom the drop down list.')
            self.ui_enable_controls()
        else:
            # clean_iso_cfg_ext_dir(os.path.join(multibootusb_host_dir(), "iso_cfg_ext_dir"))  # Need to be cleaned.
            # extract_cfg_file(config.image_path)  # Extract files from ISO
            # config.distro = distro(iso_cfg_ext_dir(), config.image_path)  # Detect supported distro
            usb_details = usb.details(config.usb_disk)
            log("MultiBoot Install: USB Disk: " + config.usb_disk)
            log("MultiBoot Install: USB Label: " + config.usb_label)
            log("MultiBoot Install: USB UUID: " + config.usb_uuid)
            log("MultiBoot Install: USB mount path: " + config.usb_mount)
            log("MultiBoot Install: Disk total size: " + str(usb.bytes2human(usb_details['size_total'])))
            log("MultiBoot Install: Disk used size: " + str(usb.bytes2human(usb_details['size_used'])))
            log("MultiBoot Install: Disk free size: " + str(usb.bytes2human(usb_details['size_free'])))
            log("MultiBoot Install: Filesystem: " + usb_details['file_system'])
            log("MultiBoot Install: Disk vendor: " + usb_details['vendor'])
            log("MultiBoot Install: Disk model: " + usb_details['model'])
            log("MultiBoot Install: ISO file: " + iso_name(config.image_path))

            if os.path.exists(config.image_path):
                # self.ui.image_path.clear()
                if config.distro:
                    log("MultiBoot Install: Distro type detected: " + config.distro)
                    if not os.path.exists(
                            os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path))):
                        config.persistence = self.ui.slider_persistence.value() * 1024 * 1024
                        log("Persistence chosen is " + str(bytes2human(config.persistence)))
                        install_size = iso_size(config.image_path) + config.persistence
                        if install_size >= disk_usage(config.usb_mount).free:
                            log("ERROR: Not enough space available on " + config.usb_disk)
                            QtWidgets.QMessageBox.information(self, "No Space.",
                                                              "No space available on " + config.usb_disk)
                            self.ui_enable_controls()
                        else:
                            if config.distro == 'memdisk_iso':
                                reply = QtWidgets.QMessageBox.question(self, 'Review selection...',
                                                                       'The ISO sleceted is not supported at the moment.\n'
                                                                       'You can try booting ISO using memdisk.\n'
                                                                       'Distro can be uninstalled anytime from main menu.\n\n'
                                                                       'Proceed with installation?',
                                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                                       QtWidgets.QMessageBox.No)
                            else:
                                reply = QtWidgets.QMessageBox.question(self, 'Review selection...',
                                                                       'Selected USB disk: %s\n' % config.usb_disk +
                                                                       'USB mount point: %s\n' % config.usb_mount +
                                                                       'Selected distro: %s\n\n' % iso_name(
                                                                           config.image_path) +
                                                                       'Proceed with installation?',
                                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                                       QtWidgets.QMessageBox.No)

                            if reply == QtWidgets.QMessageBox.Yes:
                                self.ui.slider_persistence.setEnabled(False)
                                copy_mbusb_dir_usb(config.usb_disk)
                                config.process_exist = True
                                self.progress_thread_install.start()
                            elif reply == QtWidgets.QMessageBox.No:
                                self.ui_enable_controls()

                    else:
                        QtWidgets.QMessageBox.information(self, 'Already exists...',
                                                          os.path.basename(
                                                              config.image_path) + ' is already installed.')
                        self.ui_enable_controls()
                else:
                    QtWidgets.QMessageBox.information(self, 'No support...',
                                                      'Sorry.\n' + os.path.basename(config.image_path) +
                                                      ' is not supported at the moment.\n'
                                                      'Please email this issue to feedback.multibootusb@gmail.com')
                    self.ui_enable_controls()

                    # Added to refresh usb disk remaining size after distro installation
                    # self.update_gui_usb_info()

    def dd_finished(self):
        """
        Re-enable the blocked widgets for newer use.
        :return:
        """
        self.ui.progressbar.setValue(0)
        self.ui.statusbar.showMessage("Status: Idle")
        config.process_exist = None

        msgBox = QtWidgets.QMessageBox()
        msgBox.setText("Image succesfully written to USB disk.")
        msgBox.setInformativeText("Reboot to boot from USB or test it from <b>Boot ISO/USB</b> tab.");
        msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
        msgBox.setIcon(QtWidgets.QMessageBox.Information)
        msgBox.exec_()

        self.ui_enable_controls()

    def dd_start(self):
        """
        Function to block the widgets under ISO Imager tab...
        :return:
        """
        self.ui.progressbar.setValue(0)
        self.ui.statusbar.showMessage("Status: Idle")
        # FIXME        self.ui.lineEdit_3.clear()
        self.ui.button_browse_image.setEnabled(False)
        self.ui.combo_drives.setEnabled(False)
        # FIXME self.ui.pushbtn_imager_refreshusb.setEnabled(False)
        status_text = ("Status: Writing " + os.path.basename(config.image_path) + " to " + config.usb_disk)
        self.ui.statusbar.showMessage(status_text)

    def dd_quit(self):
        self.ui.progressbar.setValue(0)
        self.ui.statusbar.showMessage("Status: Idle")
        self.ui.combo_drives.setEnabled(True)
        self.ui.button_browse_image.setEnabled(True)
        QtWidgets.QMessageBox.information(self, 'Failed!', 'Failed writing image.')

    def dd_write(self):
        self.ui_disable_controls()

        if not config.usb_disk:
            QtWidgets.QMessageBox.information(self, 'No USB disk selected',
                                              'Please insert USB disk and click "Detect Drives".')
            self.ui_enable_controls()
        elif not config.image_path:
            QtWidgets.QMessageBox.information(self, 'No ISO selected', 'Please select an ISO.')
            self.ui_enable_controls()
        else:
            imager = Imager()
            if platform.system() == 'Linux' and config.usb_details['devtype'] == "partition":
                gen.log('Selected device is a partition. Please select a disk from the drop down list')
                QtWidgets.QMessageBox.information(self, 'Incompatible device', 'Selected device (%s) is a partition!\n'
                                                                               'ISO must be written to a whole disk.'
                                                                               '\n\nPlease select a disk from the drop down list.' % config.usb_disk)
                self.ui_enable_controls()
            else:
                usb_disk_size = int(imager.imager_usb_detail(config.usb_disk, partition=0).total_size)
                self.iso_size = os.path.getsize(config.image_path)
                if self.iso_size >= usb_disk_size:
                    QtWidgets.QMessageBox.information(self, "No enough space on disk.",
                                                      os.path.basename(config.image_path) +
                                                      " size is larger than the size of " + config.usb_disk)
                    self.ui_enable_controls()
                # elif gen.process_exist('explorer.exe') is not False:
                #    # Check if windows explorer is running and inform user to close it.
                #    QtWidgets.QMessageBox.information(self, "Windows Explorer", "Windows Explorer is running\n"
                #                                                                "You need to close it before writing ISO "
                #                                                                "image to disk...")
                else:
                    reply = QtWidgets.QMessageBox.question \
                        (self, 'Review selection',
                         'Selected disk: %s\n' % config.usb_disk +
                         'Selected image: %s\n\n' % os.path.basename(config.image_path) +
                         'Proceed with writing image to disk?',
                         QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)

                    if reply == QtWidgets.QMessageBox.Yes:
                        self.dd_start()
                        config.process_exist = True
                        self.progress_thread_dd.start()
                    elif reply == QtWidgets.QMessageBox.No:
                        self.ui_enable_controls()

    def on_close_Click(self):
        """
        Closes main GUI.
        :return:
        """
        self.close()

    def closeEvent(self, event):
        """
        To capture the main close event.
        :param event: Close event.
        :return:
        """
        if config.process_exist == None:
            event.accept()
        else:
            reply = QtWidgets.QMessageBox.question(self, 'Exit MultiBootUSB...',
                                                   "A process is still running.\n"
                                                   "Do you really want to quit multibootusb?",
                                                   QtWidgets.QMessageBox.Yes,
                                                   QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.Yes:
                log("Closing multibootusb...")
                event.accept()
                sys.exit(0)
            else:
                log("Close event cancelled.")
                event.ignore()


class GuiInstallProgress(QtCore.QThread):
    """
    Update GUI thread during install.
    """
    update = QtCore.pyqtSignal(int)
    status = QtCore.pyqtSignal(str)
    finished = QtCore.pyqtSignal()

    def __init__(self):
        QtCore.QThread.__init__(self)

    def __del__(self):
        self.wait()

    def run(self):
        install_dir = os.path.join(config.usb_mount, "multibootusb", iso_basename(config.image_path))
        self.thread = GenericThread(install_progress)
        status_text = "Status: "
        self.thread.start()
        while self.thread.isRunning():
            if config.status_text.strip():
                config.status_text = config.status_text.replace(install_dir + "/", "Extracting ")
            self.update.emit(config.percentage)
            self.status.emit(config.status_text)
            if not self.thread.isFinished() and config.percentage == 100:
                config.status_text = "Status: Please wait..."
                self.status.emit("Status: Please wait...")
            time.sleep(0.1)

        self.update.emit(100)
        self.update.emit(0)

        self.status.emit("Status: Installing boot loader...")

        if self.thread.isFinished():
            config.status_text = ""
            self.finished.emit()

        log("Distro extraction completed...")

        return


class GuiUninstallProgress(QtCore.QThread):
    """
    Update GUI thread during uninstall.
    """
    update = QtCore.pyqtSignal(int)
    status = QtCore.pyqtSignal(str)
    finished = QtCore.pyqtSignal()

    def __init__(self):
        QtCore.QThread.__init__(self)
        self.thread = GenericThread(uninstall_progress)

    def __del__(self):
        self.wait()

    def run(self):
        self.thread.start()

        while self.thread.isRunning():
            self.update.emit(config.percentage)
            self.status.emit(config.status_text)
            if not self.thread.isFinished() and config.percentage == 100:
                config.status_text = "Please wait..."
            time.sleep(0.1)

        self.update.emit(100)
        self.update.emit(0)
        config.percentage = 0
        self.status.emit("Updating syslinux.cfg file...")

        if self.thread.isFinished():
            config.status_text = ""
            self.finished.emit()

        log("Distro uninstall is complete...")

        return


class DD_Progress(QtCore.QThread):
    """
    Update GUI progress bar without blocking rest of GUI element when dd process is in progress.
    """
    update = QtCore.pyqtSignal(int)
    status = QtCore.pyqtSignal(str)
    finished = QtCore.pyqtSignal()

    def __init__(self):
        QtCore.QThread.__init__(self)

        if platform.system() == 'Linux':
            self.thread = GenericThread(dd_linux)
        elif platform.system() == 'Windows':
            self.thread = GenericThread(dd_win)

    def __del__(self):
        self.wait()

    def run(self):
        self.thread.start()
        while self.thread.isRunning():
            if config.imager_percentage:
                self.update.emit(config.imager_percentage)
            if not self.thread.isFinished() and config.percentage == 100:
                config.imager_status_text = ""
                self.status.emit("Please wait...")
            time.sleep(0.1)

        self.update.emit(100)
        self.update.emit(0)

        if self.thread.isFinished():
            config.status_text = ""
            self.finished.emit()

        return


class GenericThread(QtCore.QThread):
    def __init__(self, function, *args, **kwargs):
        QtCore.QThread.__init__(self)
        self.function = function
        self.args = args
        self.kwargs = kwargs

    def __del__(self):
        self.wait()

    def run(self):
        self.function(*self.args, **self.kwargs)
        return


def show_admin_info():
    """
    Show simple information box reminding user to run the software with admin/root privilege.
    Only required under Linux as the windows executable always will start with admin privilege.
    :return:
    """
    msg = QtWidgets.QMessageBox()
    msg.setIcon(QtWidgets.QMessageBox.Information)
    if os.path.exists('scripts'):
        msg.setText('Admin privilege is required to run multibootusb.\n'
                    'Try  \'sudo python3 ./multibootusb\'\n')
    else:
        msg.setText('Admin privilege is required to run multibootusb.\n'
                    'Try  \'multibootusb-pkexec\'\n')
    msg.exec_()


def main_gui():
    app = QtWidgets.QApplication(sys.argv)
    #    ui_about = Ui_About()
    #    ui = Ui_MainWindow()

    if platform.system() == 'Linux' and os.getuid() != 0:
        show_admin_info()
        sys.exit(2)

    else:
        window = AppGui()
        window.show()
        window.setWindowTitle("MultiBootUSB - " + mbusb_version())
        window.setWindowIcon(QtGui.QIcon(resource_path(os.path.join("data", "tools", "multibootusb.png"))))
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

# pylint: disable=anomalous-backslash-in-string

"""pyudev.wx
    =========

    Wx integration.

    :class:`MonitorObserver` integrates device monitoring into the wxPython\_
    mainloop by turing device events into wx events.

    :mod:`wx` from wxPython\_ must be available when importing this module.

    .. _wxPython: http://wxpython.org/

    .. moduleauthor::  Tobias Eberle  <tobias.eberle@gmx.de>
    .. versionadded:: 0.14

"""


from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from wx import EvtHandler, PostEvent
from wx.lib.newevent import NewEvent

import pyudev


DeviceEvent, EVT_DEVICE_EVENT = NewEvent()


class MonitorObserver(EvtHandler):
    """
    An observer for device events integrating into the :mod:`wx` mainloop.

    This class inherits :class:`~wx.EvtHandler` to turn device events into
    wx events:

    >>> from pyudev import Context, Monitor
    >>> from pyudev.wx import MonitorObserver
    >>> context = Context()
    >>> monitor = Monitor.from_netlink(context)
    >>> monitor.filter_by(subsystem='input')
    >>> observer = MonitorObserver(monitor)
    >>> def device_event(event):
    ...     print('action {0} on device {1}'.format(event.device.action, event.device))
    >>> observer.Bind(EVT_DEVICE_EVENT, device_event)
    >>> monitor.start()

    This class is a child of :class:`wx.EvtHandler`.

    .. versionadded:: 0.17
    """

    def __init__(self, monitor):
        EvtHandler.__init__(self)
        self.monitor = monitor
        self._observer_thread = None
        self.start()

    @property
    def enabled(self):
        """
        Whether this observer is enabled or not.

        If ``True`` (the default), this observer is enabled, and emits events.
        Otherwise it is disabled and does not emit any events.
        """
        return self._observer_thread is not None

    @enabled.setter
    def enabled(self, value):
        if value:
            self.start()
        else:
            self.stop()

    def start(self):
        """
        Enable this observer.

        Do nothing, if the observer is already enabled.
        """
        if self._observer_thread is not None:
            return
        self._observer_thread = pyudev.MonitorObserver(
            self.monitor, callback=self._emit_event,
            name='wx-observer-thread')
        self._observer_thread.start()

    def stop(self):
        """
        Disable this observer.

        Do nothing, if the observer is already disabled.
        """
        if self._observer_thread is None:
            return
        self._observer_thread.stop()

    def _emit_event(self, device):
        PostEvent(self, DeviceEvent(device=device))


DeviceAddedEvent, EVT_DEVICE_ADDED = NewEvent()
DeviceRemovedEvent, EVT_DEVICE_REMOVED = NewEvent()
DeviceChangedEvent, EVT_DEVICE_CHANGED = NewEvent()
DeviceMovedEvent, EVT_DEVICE_MOVED = NewEvent()


class WxUDevMonitorObserver(MonitorObserver):
    """An observer for device events integrating into the :mod:`wx` mainloop.

    .. deprecated:: 0.17
       Will be removed in 1.0.  Use :class:`MonitorObserver` instead.
    """

    _action_event_map = {
        'add': DeviceAddedEvent,
        'remove': DeviceRemovedEvent,
        'change': DeviceChangedEvent,
        'move': DeviceMovedEvent
    }

    def __init__(self, monitor):
        MonitorObserver.__init__(self, monitor)
        import warnings
        warnings.warn('Will be removed in 1.0. '
                      'Use pyudev.wx.MonitorObserver instead.',
                      DeprecationWarning)

    def _emit_event(self, device):
        PostEvent(self, DeviceEvent(action=device.action, device=device))
        event_class = self._action_event_map.get(device.action)
        if event_class is not None:
            PostEvent(self, event_class(device=device))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     menus.py
# Purpose:  Module contain custom menu entries for fewer distros
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

from . import iso
from . import config
from .gen import MemoryCheck


def pc_tool_config(syslinux=True, grub=False):
    """
    Menu entry for PC Tool ISO
    :param syslinux:
    :param grub:
    :return:
    """
    if syslinux is True:
        return """KERNEL /system/stage1
APPEND initrd=/system/stage2 root=/dev/ram0 rw rdinit=/linuxrc video=vesa:ywrap,mtrr vga=0x303 loglevel=0 splash boot=cdrom\n"""
    elif grub is True:
        return """menuentry """ + iso.iso_basename(config.image_path) + """ {
linux /system/stage1 root=/dev/ram0 rw rdinit=/linuxrc video=vesa:ywrap,mtrr vga=0x303 loglevel=0 splash boot=cdrom
initrd /system/stage2\n}"""


def grub2only():
    return """MENU LABEL """ + iso.iso_basename(config.image_path) + """
LINUX /multibootusb/grub/lnxboot.img
INITRD /multibootusb/grub/core.img
TEXT HELP
    Switch to GRUB2 to select boot options.
ENDTEXT\n"""


def rising(syslinux=True, grub=False):
    """
    Menu entry for Raising Anti-virus. Only grub2 menu is required. Syslinux menu is automatically added 
    :param syslinux: 
    :param grub: 
    :return: 
    """
    if grub is True:
        return """menuentry """ + iso.iso_basename(config.image_path) + """ {
linux /multibootusb/linux/boot/isolinux/vmlinuz lang=us ramdisk_size=100000 init=/etc/init apm=power-off pnpbios=off vga=0x314 nomce quiet BOOT_IMAGE=rising 
initrd /multibootusb/linux/boot/isolinux/ravroot.gz\n}"""


def memdisk_iso_cfg(syslinux=True, grub=False):
    """
    Menu entry for ISO using memdisk. Need to have more RAM to make it work with mendisk properly.
    By default, memory size is set to system RAM size or 3GB. The 'vmalloc=3072M' can also be used as 'mem=3G'.
    :param syslinux: Default is True
    :param grub: Default is False
    :return: Menu entry for syslinux and grub2
    """
    M = MemoryCheck()
    avl_mem = M.value
    if not avl_mem:
        avl_mem = int(3072)
    if syslinux is True:
        menu_entry = 'LINUX memdisk\n' \
        'INITRD /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
        'APPEND iso vmalloc=' + str(avl_mem) + 'M\n'
    elif grub is True:
        menu_entry = '    search --set -f /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
                     '    menuentry ' + iso.iso_basename(config.image_path) + ' {\n' \
                     '    linux16 /multibootusb/memdisk iso vmalloc=' + str(avl_mem) + 'M\n' \
                     '    initrd16 /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
                     '}\n'

    return menu_entry


def memdisk_img_cfg(syslinux=True, grub=False):
    """
    Menu entry for IMG files using memdisk. Need to have more RAM to make it work with mendisk properly.
    By default, memory size is set to system RAM size or 3GB.
    :param syslinux: Default is True
    :param grub: Default is False
    :return: Menu entry for syslinux and grub2
    """
    M = MemoryCheck()
    avl_mem = M.value
    if not avl_mem:
        avl_mem = int(3072)
    if syslinux is True:
        menu_entry = 'LINUX memdisk\n' \
        'INITRD /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
        'APPEND raw vmalloc=' + str(avl_mem) + 'M\n'
    elif grub is True:
        menu_entry = '    search --set -f /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
                     '    menuentry ' + iso.iso_basename(config.image_path) + ' {\n' \
                     '    linux16 /multibootusb/memdisk raw vmalloc=' + str(avl_mem) + 'M\n' \
                     '    initrd16 /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
                     '}\n'

    return menu_entry





#!/usr/bin/env python3
# -*- coding: utf-8; mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vim: fileencoding=utf-8 tabstop=4 expandtab shiftwidth=4
# Name:     admin.py
# Purpose:  Module to ask for admin rights under Linux and Windows
# Authors:  Originally developed by Preston Landers (for windows) and modified for multibootusb by Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of the same license as Python 2.6.5

##
## (C) COPYRIGHT © Preston Landers 2010
## Released under the same license as Python 2.6.5
##

"""
User Access Control for Microsoft Windows Vista and higher.  This is
only for the Windows platform.

This will relaunch either the current script - with all the same command
line parameters - or else you can provide a different script/program to
run.  If the current user doesn't normally have admin rights, he'll be
prompted for an admin password. Otherwise he just gets the UAC prompt.

This is meant to be used something like this::

    if not pyuac.isUserAdmin():
        return pyuac.runAsAdmin()

    # otherwise carry on doing whatever...

See L{runAsAdmin} for the main interface.

"""
import os
import traceback
import types
import platform
import sys
import subprocess
from PyQt5 import QtWidgets
from . import gen


def isUserAdmin():
    """
    @return: True if the current user is an 'Admin' whatever that means
    (root on Unix), otherwise False.

    Warning: The inner function fails unless you have Windows XP SP2 or
    higher. The failure causes a traceback to be gen.loged and this
    function to return False.
    """

    if platform.system() == "Windows":
        import ctypes
        # WARNING: requires Windows XP SP2 or higher!
        try:
            return ctypes.windll.shell32.IsUserAnAdmin()
        except:
            traceback.print_exc()
            gen.log("Admin check failed, assuming not an admin.")
            return False
    elif platform.system() == "Linux":
        return os.getuid() == 0
    else:
        raise RuntimeError("Unsupported operating system for this module: %s" % (os.name,))


def runAsAdmin(cmdLine=None, wait=True):
    """
    Attempt to relaunch the current script as an admin using the same
    command line parameters.  Pass cmdLine in to override and set a new
    command.  It must be a list of [command, arg1, arg2...] format.

    Set wait to False to avoid waiting for the sub-process to finish. You
    will not be able to fetch the exit code of the process if wait is
    False.

    Returns the sub-process return code, unless wait is False in which
    case it returns None.

    @WARNING: this function only works on Windows.
    """

    #if os.name == 'nt':
    #    raise RuntimeError, "This function is only implemented on Windows."
    if platform.system() == "Windows":

        import win32api
        import win32con
        import win32event
        import win32process
        from win32com.shell.shell import ShellExecuteEx
        from win32com.shell import shellcon

        python_exe = sys.executable

        if cmdLine is None:
            cmdLine = [python_exe] + sys.argv
        elif type(cmdLine) not in (types.TupleType, types.ListType):
            raise ValueError("cmdLine is not a sequence.")
        cmd = '"%s"' % (cmdLine[0],)
        # XXX TODO: isn't there a function or something we can call to massage command line params?
        params = " ".join(['"%s"' % (x,) for x in cmdLine[1:]])
#         cmdDir = ''
        showCmd = win32con.SW_SHOWNORMAL
        #showCmd = win32con.SW_HIDE
        lpVerb = 'runas'  # causes UAC elevation prompt.

        #gen.log("Running", cmd, params)

        # ShellExecute() doesn't seem to allow us to fetch the PID or handle
        # of the process, so we can't get anything useful from it. Therefore
        # the more complex ShellExecuteEx() must be used.

        # procHandle = win32api.ShellExecute(0, lpVerb, cmd, params, cmdDir, showCmd)

        procInfo = ShellExecuteEx(nShow=showCmd,
                                  fMask=shellcon.SEE_MASK_NOCLOSEPROCESS,
                                  lpVerb=lpVerb,
                                  lpFile=cmd,
                                  lpParameters=params)

        if wait:
            procHandle = procInfo['hProcess']
            obj = win32event.WaitForSingleObject(procHandle, win32event.INFINITE)
            rc = win32process.GetExitCodeProcess(procHandle)
            #gen.log "Process handle %s returned code %s" % (procHandle, rc)
        else:
            rc = None

        return rc


def adminCmd(cmd, fork=False, gui=False):
    """
    This simple function checks for a sudo command and runs a command using it.
    This function tries to launch given script with root access using pkexec/gksu/gksudo/kdesu/kdesudo,
    if one of them is already installed.
    PyQt4 is used as GUI.
    Author : sundar
    """
    sudo_cmd = ''
    if os.getuid() == 0:
        sudo_cmd = cmd
    else:
        if os.system('which pkexec') == 0:
            if gui:
                # By default, pkexec disallows X11 apps. Restore DISPLAY & XAUTHORITY
                # to allow it. man 1 pkexec/"SECURITY NOTES" section
                cmd = ['export DISPLAY=$DISPLAY; export XAUTHORITY=$XAUTHORITY; '] + cmd
            sudo_cmd = ['pkexec', '/bin/sh', '-c']
        elif os.system('which gksudo') == 0:
            sudo_cmd = ["gksudo", "--", "/bin/sh", "-c"]
        elif os.system('which gksu') == 0:
            sudo_cmd = ["gksu"]
        elif os.system('which kdesudo') == 0:
            sudo_cmd = ["kdesudo", "-t", "-c"]    # http://www.unix.com/man-page/debian/1/kdesudo/
        elif os.system('which kdesu') == 0:
            sudo_cmd = ["kdesu", "-t", "-c"]      # http://linux.die.net/man/1/kdesu
        else:
            QtWidgets.QMessageBox.information('No root...',
                                          'Could not find any of: pkexec, sudo, gksu, kdesu, gksudo, or kdesudo.\n'
                                          'Please install one then restart multibootusb.')
            sys.exit(0)
    final_cmd = ' '.join(sudo_cmd + ['"' + ' '.join(cmd).replace('"', '\\"') + '"'])
    gen.log("Executing ==>  " + final_cmd)
    if fork:
        return subprocess.Popen(final_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, bufsize=1, shell=True)
    else:
        ret = subprocess.call(final_cmd, shell=True)
        gen.log("Process returned ==>   " + str(ret))
        return ret



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     install.py
# Purpose:  Script to install multibootusb from source on different linux distros. This also pulls in dependencies.
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import sys
import urllib.request
import urllib.error
import urllib.parse
import subprocess


if not os.getuid() == 0:
    print("You must run this file with admin privilege.")
    print("Try sudo ./install.py")
    sys.exit(0)


class Install():

    def mbusb(self):
        try:
#             from PyQt5 import QtGui
            if subprocess.call("python3 setup.py install --record ./.install_files.txt", shell=True) == 0:
                    print("Installation finished.")
                    print("Find multibootusb under system menu or run from terminal  using the following command...")
                    print("\nmultibootusb\n")
                    print("You can uninstall multibootusb at any time using follwing command (with root/sudo previlage)")
                    print("\n./uninstall.sh\n")
        except:
            print("Installing missing package.")
            if self.supported_pac_manager() is not True:
                print("Unsupported package manager.")
                print("Please install parted, util-linux and python3-pyqt5/PyQt5, mtools and python3-dbus\n"
                      "Whatever the package name is applicable to your distro and rerun this script.")
                sys.exit(0)
            elif self.internet_on() is False:
                print("Unable to connect to internet.")
                print("Please install parted, util-linux and python3-pyqt5/PyQt5, pkexec, mtools and python3-dbus \n"
                      "Whatever the package name is applicable to your distro and rerun this script.")
                sys.exit(0)
            elif self.internet_on() is True:
                if self.install_dependency_package() is not True:
                    print("Error installing dependency packages.")
                else:
                    if subprocess.call("python3 setup.py install --record ./.install_files.txt", shell=True) == 0:
                        print("Installation finished.")
                        print("Find multibootusb under system menu or run from terminal  using the following command...")
                        print("\nmultibootusb\n")
                        print("You can uninstall multibootusb at any time using follwing command (with root/sudo previlage)")
                        print("\nsudo ./uninstall.sh\n")

    @staticmethod
    def internet_on():
        try:
            ret = urllib.request.urlopen('https://www.google.com', timeout=1)
            print("Interconnection exist.")
            result = True

        except urllib.error.URLError:
            print("Interconnection does not exist.")
            result = False

        return result

    @staticmethod
    def supported_pac_manager():
        pac_managers = ["pacman", "yum", "apt-get", "zypper", "urpmi"]
        result = "0"
        for pac_man in pac_managers:
            if subprocess.call("which " + pac_man, shell=True) == 0:
                result = "1"
                return True

        if not result == "1":
            return False


    @staticmethod
    def install_dependency_package():
        if subprocess.call("which pacman", shell=True) == 0:
            subprocess.call("pacman -Sy --noconfirm", shell=True)
            # Thank you Neitsab for "--needed"  argument.
            if subprocess.call("pacman -S --needed --noconfirm p7zip python-pyqt5 mtools python3-six parted util-linux python-dbus") == 0:
                result = True
        elif subprocess.call("which yum", shell=True) == 0:
            subprocess.call("yum check-update", shell=True)
            if subprocess.call("dnf install mtools python3-PyQt5 util-linux python3-six parted p7zip p7zip-plugins python3-pyudev python3-dbus -y", shell=True) == 0:
                result = True
        elif subprocess.call("which apt-get", shell=True) == 0:
            subprocess.call("apt-get -q update", shell=True)
            if subprocess.call("apt-get -q -y install python3-pyqt5 p7zip-full parted util-linux python3-pyudev mtools python3-dbus", shell=True) == 0:
                result = True
        elif subprocess.call("which zypper", shell=True) == 0:
            subprocess.call("zypper refresh", shell=True)
            if subprocess.call("zypper install -y mtools python3-qt5 p7zip python3-pyudev python3-six util-linux parted", shell=True) == 0:
                result = True
        elif subprocess.call("which urpmi", shell=True) == 0:
            subprocess.call("urpmi.update -a", shell=True)
            if subprocess.call("urpmi install -auto mtools util-linux p7zip python3-pyudev python3-six parted python3-qt5", shell=True) == 0:
                result = True

        return bool(result)

install = Install()

install.mbusb()



# -*- coding: utf-8 -*-
# Copyright (C) 2013 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""
    pyudev._ctypeslib.utils
    =======================

    Utilities for loading ctypeslib.

    .. moduleauthor::  Anne Mulhern  <amulhern@redhat.com>
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ctypes import CDLL
from ctypes.util import find_library


def load_ctypes_library(name, signatures, error_checkers):
    """
    Load library ``name`` and return a :class:`ctypes.CDLL` object for it.

    :param str name: the library name
    :param signatures: signatures of methods
    :type signatures: dict of str * (tuple of (list of type) * type)
    :param error_checkers: error checkers for methods
    :type error_checkers: dict of str * ((int * ptr * arglist) -> int)

    The library has errno handling enabled.
    Important functions are given proper signatures and return types to support
    type checking and argument conversion.

    :returns: a loaded library
    :rtype: ctypes.CDLL
    :raises ImportError: if the library is not found
    """
    library_name = find_library(name)
    if not library_name:
        raise ImportError('No library named %s' % name)
    lib = CDLL(library_name, use_errno=True)
    # Add function signatures
    for funcname, signature in signatures.items():
        function = getattr(lib, funcname, None)
        if function:
            argtypes, restype = signature
            function.argtypes = argtypes
            function.restype = restype
            errorchecker = error_checkers.get(funcname)
            if errorchecker:
                function.errcheck = errorchecker
    return lib



#!/usr/bin/env python3
# -*- coding: utf-8 -*
# Name:     grub.py
# Purpose:  This module contain many functions used for updating grub.cfg file to provide support for EFI/UEFI booting
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above
import os
import re
from . import config
from . import iso
from . import _7zip
from . import gen
from .usb import bytes2human
from . import menus


def mbusb_update_grub_cfg():
    """
    Function to update grub.cfg file to support UEFI/EFI systems
    :return:
    """
    # Lets convert syslinux config file to grub2 accepted file format.
    _iso_dir = os.path.join(config.usb_mount, 'multibootusb', iso.iso_basename(config.image_path))

    # First write custom loopback.cfg file so as to be detected by iso2grub2 function later.
    write_custom_gurb_cfg()

    # Try to generate loopback entry file from syslinux config files
    try:
        gen.log('Trying to create loopback.cfg')
        iso2_grub2_cfg = iso2grub2(_iso_dir)
    except Exception as e:
        print(e)
        gen.log(e)
        gen.log('Error converting syslinux cfg to grub2 cfg', error=True)
        iso2_grub2_cfg = False

    grub_cfg_path = None
    syslinux_menu = None
#     sys_cfg_path = None
    loopback_cfg_path = None
    mbus_grub_cfg_path = os.path.join(config.usb_mount, 'multibootusb', 'grub', 'grub.cfg')
#     iso_grub_cfg = iso.iso_file_path(config.image_path, 'grub.cfg')
    if iso.isolinux_bin_dir(config.image_path) is not False:
        iso_sys_cfg_path = os.path.join(iso.isolinux_bin_dir(config.image_path), 'syslinux.cfg')
        iso_iso_cfg_path = os.path.join(iso.isolinux_bin_dir(config.image_path), 'isolinux.cfg')

        if os.path.exists(os.path.join(config.usb_mount, 'multibootusb', iso.iso_basename(config.image_path),
                                       iso_sys_cfg_path)):
            syslinux_menu = iso_sys_cfg_path.replace('\\', '/')
        elif os.path.exists(os.path.join(config.usb_mount, 'multibootusb', iso.iso_basename(config.image_path),
                                         iso_iso_cfg_path)):
            syslinux_menu = iso_iso_cfg_path.replace('\\', '/')

    efi_grub_cfg = get_grub_cfg(config.image_path)
    boot_grub_cfg = get_grub_cfg(config.image_path, efi=False)
    loopback_cfg_path = iso.iso_file_path(config.image_path, 'loopback.cfg')

    if loopback_cfg_path is not False:
        grub_cfg_path = loopback_cfg_path.replace('\\', '/')
    elif efi_grub_cfg is not False:
        grub_cfg_path = efi_grub_cfg.replace('\\', '/')
    elif boot_grub_cfg is not False:
        grub_cfg_path = boot_grub_cfg.replace('\\', '/')
    elif iso2_grub2_cfg is not False:
        grub_cfg_path = iso2_grub2_cfg.replace('\\', '/')
    #elif bootx_64_cfg is not False:
    #    grub_cfg_path = bootx_64_cfg.replace('\\', '/')

    if os.path.exists(mbus_grub_cfg_path):
        gen.log('Updating grub.cfg file...')
        if grub_custom_menu(mbus_grub_cfg_path, config.distro) is False:
            with open(mbus_grub_cfg_path, 'a') as f:
                f.write("#start " + iso.iso_basename(config.image_path) + "\n")
                if grub_cfg_path is not None:
                    if  config.distro == 'grub2only':
                        f.write('     menuentry ' + iso.iso_basename(config.image_path) + ' {configfile '
                                + '/' + grub_cfg_path.replace('\\', '/') + '}' + "\n")
                    else:
                        f.write('     menuentry ' + iso.iso_basename(config.image_path) + ' {configfile '
                                + '/multibootusb/' + iso.iso_basename(config.image_path) + '/' + grub_cfg_path + '}' + "\n")
                elif config.distro == 'f4ubcd':
                    f.write('     menuentry ' + iso.iso_basename(config.image_path) +
                            ' {linux /multibootusb/grub.exe --config-file=/multibootusb' +
                            iso.iso_basename(config.image_path) + '/menu.lst}'"\n")
                elif config.distro == 'pc-unlocker':
                    f.write('     menuentry ' + iso.iso_basename(config.image_path) +
                            ' {\n    linux /ldntldr\n    ntldr /ntldr }' + "\n")
                elif config.distro == 'ReactOS':
                    f.write('     menuentry ' + iso.iso_basename(config.image_path) +
                            ' {multiboot /loader/setupldr.sys}' + "\n")
                elif config.distro == 'memdisk_img':
                    f.write(menus.memdisk_img_cfg(syslinux=False, grub=True))
                elif config.distro == 'memdisk_iso':
                    f.write(menus.memdisk_iso_cfg(syslinux=False, grub=True))
                elif config.distro == 'memtest':
                    f.write('     menuentry ' + iso.iso_basename(config.image_path) +
                            ' {linux16 ' + '/multibootusb/' + iso.iso_basename(config.image_path) + '/BISOLINUX/MEMTEST}' + "\n")
                elif syslinux_menu is not None:
                    f.write('     menuentry ' + iso.iso_basename(config.image_path) + ' {syslinux_configfile '
                            + '/multibootusb/' + iso.iso_basename(config.image_path) + '/' + syslinux_menu + '}' + "\n")
                f.write("#end " + iso.iso_basename(config.image_path) + "\n")

    # Ascertain if the entry is made..
    if gen.check_text_in_file(mbus_grub_cfg_path, iso.iso_basename(config.image_path)):
        gen.log('Updated entry in grub.cfg...')
    else:
        gen.log('Unable to update entry in grub.cfg...')


def write_custom_gurb_cfg():
    """
    Create custom grub loopback.cfg file for known distros. Custom menu entries are stored on munus.py module
    :return: 
    """
    loopback_cfg_path = os.path.join(config.usb_mount, 'multibootusb', iso.iso_basename(config.image_path), 'loopback.cfg')
    menu = False
    if config.distro == 'pc-tool':
        menu = menus.pc_tool_config(syslinux=False, grub=True)
    elif config.distro == 'rising-av':
        menu = menus.rising(syslinux=False, grub=True)

    if menu is not False:
        gen.log('Writing custom loopback.cfg file...')
        write_to_file(loopback_cfg_path, menu)


def get_grub_cfg(iso_link, efi=True):
    """
    Detects path to "grub.cfg" file from ISO file. Default is to get from EFI directory.
    :return: path of "grub.cfg" file as string.
    """
    if os.path.exists(iso_link):
        grub_path = False
        iso_file_list = _7zip.list_iso(iso_link)
        if any("grub" in s.lower() for s in iso_file_list):
            for f in iso_file_list:
                f_basename = os.path.basename(f).lower()
                if f_basename.startswith('grub') and f_basename.endswith('.cfg'):
                #if 'grub.cfg' in f.lower():
                    if efi is True:
                        if 'efi' in f.lower():
                            grub_path = f.replace('\\', '/')
                            gen.log('Found ' + grub_path)
                            break
                        elif 'boot' in f.lower():
                            grub_path = f.replace('\\', '/')
                            gen.log('Found ' + grub_path)
                            break
                        else:
                            grub_path = f.replace('\\', '/')
                            gen.log('Found ' + grub_path)
                            break
        return grub_path


def grub_custom_menu(mbus_grub_cfg_path, distro):
    iso_size_mb = bytes2human(iso.iso_size(config.image_path))
    gen.log('size of the ISO is ' + str(iso_size_mb))
    if distro in ['sgrubd2', 'raw_iso']:
        grub_raw_iso(mbus_grub_cfg_path)

#         with open(mbus_grub_cfg_path, 'a') as f:
#             f.write("#start " + iso.iso_basename(config.image_path) + "\n")
#             f.write(grub_raw_iso())
#             f.write("#end " + iso.iso_basename(config.image_path) + "\n")
#
#
#     elif iso_size_mb < 750.0:
#         grub_raw_iso(mbus_grub_cfg_path)

    else:
        return False


def grub_raw_iso(mbus_grub_cfg_path):
    """
    Generic menu entry for booting ISO files directly using memdisk. Should have enough memory to load to RAM
    :return:
    """
    menu_entry = '    search --set -f /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
                 '    menuentry ' + iso.iso_basename(config.image_path) + ' {\n' \
                 '    linux16 /multibootusb/memdisk iso raw vmalloc=750M\n' \
                 '    initrd16 /multibootusb/' + iso.iso_basename(config.image_path) + '/' + iso.iso_name(config.image_path) + '\n' \
                 '}\n'
    with open(mbus_grub_cfg_path, 'a') as f:
        f.write("#start " + iso.iso_basename(config.image_path) + "\n")
        f.write(menu_entry)
        f.write("#end " + iso.iso_basename(config.image_path) + "\n")
    return menu_entry


def string_in_file(_file, search_text):
    """
    Search if string exist in a file.
    :param _file: Path to file
    :param search_text: String to be searched
    :return: True if string is found
    """
    if search_text in open(_file).read().lower():
        return True


def write_to_file(file_path, _strings):

    try:
        if not os.path.exists(file_path):
            open(file_path, 'a').close()

        with open(file_path, 'a') as f:
            f.write(_strings + '\n')
    except:
        gen.log('Error writing to loopback.cfg file..')


def extract_kernel_line(search_text, match_line, isolinux_dir):
    """
    Function to check if kernel/linux line present in isolinux.cfg file is valid.
    If valid, then convert them in to grub accepted format
    :param search_text: Type of text is to be searched. Typically kernel or linux
    :param match_line: Line containing kernel ot linux from isolinux supported .cfg files
    :param isolinux_dir: Path to isolinux directory of an ISO
    :return: Valid grub2 accepted kernel/linux line after conversion. If nothing found return ''.
    """
    kernel_line = ''

    # Remove '=' from linux/kernel parameter
    if (search_text + '=') in match_line:
        kernel_path = match_line.replace((search_text + '='), '').strip()
        search_text = search_text.replace('=', '')
    else:
        kernel_path = match_line.replace(search_text, '', 1).strip()

    # Check if kernel/linux exist using absolute path (kernel_path)
    if os.path.exists(os.path.join(config.usb_mount, kernel_path)):
        kernel_line = search_text.lower().replace('kernel', 'linux') + ' ' + kernel_path.strip()

    # Check if path to kernel/linux exist in isolinux directory and return absolute path
    elif os.path.exists(os.path.join(config.usb_mount, 'multibootusb', iso.iso_basename(config.image_path), isolinux_dir, kernel_path)):
        kernel_line = search_text.lower().replace('kernel', 'linux') + ' /multibootusb/' + \
                      iso.iso_basename(config.image_path) + '/' + isolinux_dir + '/' + kernel_path.strip()

    # Check if multiple kernel/linux exist and convert it to grub accepted line
    # Found such entries in manjaro and slitaz
    elif ',/' in kernel_path:
        kernel_line = search_text.lower().replace('kernel', 'linux') + ' ' + kernel_path.strip()
        kernel_line = kernel_line.replace(',/', ' /')

    # Same as above but I found this only in dban
    elif 'z,' in kernel_path:
        kernel_line = search_text.lower().replace('kernel', 'linux') + ' ' + kernel_path.strip()
        kernel_line = kernel_line.replace('z,', ' /multibootusb/' + iso.iso_basename(config.image_path) + '/'
                                          + iso.isolinux_bin_dir(config.image_path).replace('\\', '/') + '/')
    else:
        kernel_line = ''

    return kernel_line.replace('\\', '/').replace('//', '/')


def iso2grub2(iso_dir):
    """
    Function to convert syslinux configuration to grub2 accepted configuration format. Features implemented are similar
    to that of grub2  'loopback.cfg'. This 'loopback.cfg' file can be later on caled directly from grub2. The main
    advantage of this function is to generate the 'loopback.cfg' file automatically without manual involvement.
    :param iso_dir: Path to distro install directory for looping through '.cfg' files.
    :param file_out: Path to 'loopback.cfg' file. By default it is set to root of distro install directory.
    :return:
    """
    grub_file_path = os.path.join(config.usb_mount, 'multibootusb', iso.iso_basename(config.image_path), 'loopback.cfg')
    gen.log('loopback.cfg file is set to ' + grub_file_path)
    iso_bin_dir = iso.isolinux_bin_dir(config.image_path)
    # Loop though the distro installed directory for finding config files
    for dirpath, dirnames, filenames in os.walk(iso_dir):
        for f in filenames:
            # We will strict to only files ending with '.cfg' extension. This is the file extension isolinux or syslinux
            #  recommends for writing configurations
            if f.endswith((".cfg", ".CFG")):
                cfg_file_path = os.path.join(dirpath, f)
                # We will omit the grub directory
                if 'grub' not in cfg_file_path:
                    # we will use only files containing strings which can be converted to grub2 cfg style
                    if string_in_file(cfg_file_path, 'menu label') or string_in_file(cfg_file_path, 'label'):
                        with open(cfg_file_path, "r", errors='ignore') as cfg_file_str:
                            data = cfg_file_str.read()
                            # Make sure that lines with menu label, kernel and append are available for processing
                            ext_text = re.finditer('(menu label|label)(.*?)(?=(menu label|label))', data, re.I|re.DOTALL)
                            if ext_text:
                                for m in ext_text:
                                    menuentry = ''
#                                     kernel = ''
                                    kernel_line = ''
                                    boot_options = ''
                                    initrd_line = ''
#                                     initrd = ''

                                    # Extract line containing 'menu label' and convert to menu entry of grub2
                                    if 'menu label' in m.group().lower():
                                        menu_line = re.search('menu label(.*)\s', m.group(), re.I).group()
                                        menuentry = 'menuentry ' + gen.quote(re.sub(r'menu label', '', menu_line, re.I, ).strip())
                                        # Ensure that we do not have caps menu label in the menuentry
                                        menuentry = menuentry.replace('MENU LABEL', '')

                                    elif 'label ' in m.group().lower():
                                        # Probably config does not use 'menu label' line. Just line containing 'label'
                                        #  and convert to menu entry of grub2
                                        menu_line = re.search('^label(.*)\s', m.group(), re.I).group()
                                        menuentry = 'menuentry ' + gen.quote(re.sub(r'label', '', menu_line, re.I, ).strip())
                                        # Ensure that we do not have caps label in the menuentry
                                        menuentry = menuentry.replace('LABEL', '')

                                    # Extract kernel line and change to linux line of grub2
                                    if 'kernel' in m.group().lower() or 'linux' in m.group().lower():
                                        kernel_text = re.findall('((kernel|linux)[= ].*?[ \s])', m.group(), re.I)
                                        match_count = len(re.findall('((kernel|linux)[= ].*?[ \s])', m.group(), re.I))
                                        if match_count is 1:
                                            kernel_line = extract_kernel_line(kernel_text[0][1], kernel_text[0][0], iso_bin_dir)
                                        elif match_count > 2:
                                            for _lines in kernel_text:
                                                kernel_line = extract_kernel_line(_lines[0][1], _lines[0][0],
                                                                                  iso_bin_dir)
                                                if kernel_line == '':
                                                    continue
                                                else:
                                                    break

                                    if 'initrd' in m.group().lower():
                                        initrd_text = re.findall('((initrd)[= ].*?[ \s])', m.group(), re.I)
                                        match_count = len(re.findall('((initrd)[= ].*?[ \s])', m.group(), re.I))
                                        if match_count is 1:
                                            initrd_line = extract_kernel_line(initrd_text[0][1], initrd_text[0][0], iso_bin_dir)
                                        elif match_count > 2:
                                            for _lines in initrd_text:
                                                initrd_line = extract_kernel_line(_lines[0][1], _lines[0][0],
                                                                                  iso_bin_dir)
                                                if initrd_line == '':
                                                    continue
                                                else:
                                                    break

                                    if 'append' in m.group().lower():
                                        append_line = re.search('append (.*)\s', m.group(), re.I).group()
                                        boot_options = re.sub(r'((initrd[= ])(.*?)[ ])', '', append_line, re.I, flags=re.DOTALL)
                                        boot_options = re.sub(r'append', '', boot_options, re.I).strip()
                                        boot_options = boot_options.replace('APPEND', '')

                                    if kernel_line.strip():
                                        linux = kernel_line.strip() + ' ' + boot_options.strip().strip()
                                    else:
                                        linux = ''

                                    if menuentry.strip() and linux.strip() and initrd_line.strip():
                                        write_to_file(grub_file_path, menuentry + '{')
                                        write_to_file(grub_file_path, '    ' + linux)
                                        write_to_file(grub_file_path, '    ' + initrd_line)
                                        write_to_file(grub_file_path, '}\n')
                                    elif menuentry.strip() and linux.strip():
                                        write_to_file(grub_file_path, menuentry + '{')
                                        write_to_file(grub_file_path, '    ' + linux)
                                        write_to_file(grub_file_path, '}\n')

    if os.path.exists(grub_file_path):
        gen.log(
            'loopback.cfg file is successfully created.\nYou must send this file for debugging if something goes wrong.')
        return 'loopback.cfg'
    else:
        gen.log('Failed to convert syslinux config file to loopback.cfg')
        return False



# -*- coding: utf-8 -*-
# Copyright (C) 2013 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""
    pyudev._ctypeslib.libc
    ======================

    Wrappers for libc.

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ctypes import c_int

from ._errorcheckers import check_errno_on_nonzero_return


fd_pair = c_int * 2


SIGNATURES = dict(
   pipe2=([fd_pair, c_int], c_int),
)

ERROR_CHECKERS = dict(
   pipe2=check_errno_on_nonzero_return,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     qemu.py
# Purpose:  Module to boot ISO and USB disks using QEMU.
# Depends:  QEMU must be installed under Linux for availing this feature. For windows, QEMU package is shipped
#           along with executable file
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import subprocess
import platform
from PyQt5 import QtWidgets
from .gui.ui_multibootusb import Ui_MainWindow
from .gen import *
from . import config


class Qemu(QtWidgets.QMainWindow, Ui_MainWindow):
    """
    ISO and USB booting using QEMU.
    """

    def __init__(self):
        QtWidgets.QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

    def on_Qemu_Boot_iso_Click(self):
        """
        Main function to boot a selected ISO.
        :return:
        """
#         if not self.ui.lineEdit_2.text():
        if not config.image_path:
            QtWidgets.QMessageBox.information(self, 'No ISO...', 'No ISO selected.\n\nPlease choose an ISO first.')
        else:
            qemu = self.check_qemu_exist()
            qemu_iso_link = config.image_path
            if not qemu:
                log("ERROR: ISO Boot: qemu not found!")
                QtWidgets.QMessageBox.information(self, 'No QEMU...', 'Please install qemu to use this feature.')
            else:
                ram = self.qemu_iso_ram()
                if ram:
                    ram = " -m " + ram
                else:
                    ram = ""

                cmd = qemu + ram + ' -boot d' + ' -cdrom "' + str(qemu_iso_link) + '"'
                try:
                    log("Executing ==> " + cmd)
                    subprocess.Popen(cmd, shell=True)
                except:
                    QtWidgets.QMessageBox.information(self, 'Error...', 'Error booting ISO\n'
                                                                    'Unable to start QEMU.')


    def on_Qemu_Boot_usb_Click(self):
        """
        Main function to boot a selected USB disk.
        :param usb_disk: Path to usb disk.
        :return:
        """
        qemu = self.check_qemu_exist()

        if not config.usb_disk:
            QtWidgets.QMessageBox.information(self, 'No disk...', 'No USB disk selected.\n\nPlease choose a disk first.')
        else:
            qemu = self.check_qemu_exist()
            if platform.system() == 'Linux' and config.usb_disk[-1].isdigit() is True:
                qemu_usb_disk = config.usb_disk[:-1]
            else:
                qemu_usb_disk = config.usb_disk

            if qemu is None:
                log("ERROR: USB Boot: qemu not found!")
                QtWidgets.QMessageBox.information(self, 'No QEMU...', 'Please install qemu to use this feature.')
            else:
                ram = self.qemu_usb_ram()
                if ram:
                    ram = " -m " + ram
                else:
                    ram = ""

                if platform.system() == "Windows":
                    disk_number = self.get_physical_disk_number(qemu_usb_disk)
                    parent_dir = os.getcwd()
                    os.chdir(resource_path(os.path.join("data", "tools", "qemu")))
                    cmd = quote(qemu) + ' -L . -boot c' + ram + ' -hda //./PhysicalDrive' + disk_number

                    try:
                        log("Executing ==>  " + cmd)
                        subprocess.Popen(cmd, shell=True)
                    except:
                        QtWidgets.QMessageBox.information(self, 'Error...', 'Error booting USB\nUnable to start QEMU.')
                    os.chdir(parent_dir)

                elif platform.system() == "Linux":
                    cmd = qemu + ' -hda "' + qemu_usb_disk + '"' + ram + ' -vga std'
                    try:
                        log('Executing ==> ' + cmd)
                        subprocess.Popen(cmd, shell=True)
                    except:
                        QtWidgets.QMessageBox.information(self, 'Error...', 'Error booting USB\n\nUnable to start QEMU.')

    def qemu_iso_ram(self):
        """
        Choose a ram size for ISO booting.
        :return: Ram size as string.
        """
        selected_ram = self.ui.combo_iso_boot_ram.currentText()
        log("QEMU: ISO RAM = " + selected_ram)

        if selected_ram == "Default":
            return None
        else:
            return selected_ram

    def qemu_usb_ram(self):
        """
        Choose a ram size for USB booting.
        :return: Ram size as string.
        """
        selected_ram = self.ui.combo_usb_boot_ram.currentText()
        log("QEMU: USB RAM = " + selected_ram)

        if selected_ram == "Default":
            return None
        else:
            return selected_ram

    @staticmethod
    def check_qemu_exist():
        """
        Check if QEMU is available on host system.
        :return: path to QEMU program or None otherwise.
        """
        if platform.system() == "Linux":
            if subprocess.call('which qemu-system-x86_64', shell=True) == 0:
                qemu = "qemu-system-x86_64"
            elif subprocess.call('which qemu', shell=True) == 0:
                qemu = "qemu"
            else:
                qemu = ""

        elif platform.system() == "Windows":
            qemu = resource_path(os.path.join("data", "tools", "qemu", "qemu-system-x86_64.exe"))
            log(qemu)

        if qemu:
            log("QEMU: using " + qemu)
        else:
            log("QEMU: ERROR: not found!")

        return qemu

    @staticmethod
    def get_physical_disk_number(usb_disk):
        """
        Get the physical disk number as detected ny Windows.
        :param usb_disk: USB disk (Like F:)
        :return: Disk number.
        """
        import wmi
        c = wmi.WMI()
        for physical_disk in c.Win32_DiskDrive():
            for partition in physical_disk.associators("Win32_DiskDriveToDiskPartition"):
                for logical_disk in partition.associators("Win32_LogicalDiskToPartition"):
                    if logical_disk.Caption == usb_disk:
#                         log physical_disk.Caption
#                         log partition.Caption
#                         log logical_disk.Caption
                        log("Physical Device Number is " + partition.Caption[6:-14])
                        return str(partition.Caption[6:-14])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     uninstall_distro.py
# Purpose:  Module to uninstall distros installed by multibootusb
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import re
import shutil
import threading
import platform
from .usb import *
from . import config
from . import gen


def install_distro_list():
    """
    List all distro names installed by previous install
    :return:  List of distro names as list
    """
    usb_details = details(config.usb_disk)
    config.usb_mount = usb_details['mount_point']
    sys_cfg_file = os.path.join(config.usb_mount, "multibootusb", "syslinux.cfg")

    if os.path.exists(sys_cfg_file):
        distro_list = []
        for line in open(sys_cfg_file):
            if "#start " in line:
                distro_list.append(line[7:])
        return distro_list
    else:
        return None


def unin_distro():
    usb_details = details(config.usb_disk)
    usb_mount = usb_details['mount_point']
    config.uninstall_distro_dir_name = config.uninstall_distro_dir_name.replace('\n', '')
    gen.log(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "multibootusb.cfg"))
    if os.path.exists(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "multibootusb.cfg")):
        with open(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "multibootusb.cfg"), "r") as multibootusb_cfg:
            config.distro = multibootusb_cfg.read().replace('\n', '')
        if config.distro:
            uninstall_distro()
    else:
        return ""


def delete_frm_file_list():
    """
    Generic way to remove files from USB disk.
    :param config.usb_disk:
    :param iso_file_list: List of files installed in the USB disk
    :param config.uninstall_distro_dir_name: Directory where the distro is installed
    :return:
    """
    usb_details = details(config.usb_disk)
    usb_mount = usb_details['mount_point']
    if config.iso_file_list is not None:
        for f in config.iso_file_list:
            if platform.system() == "Windows":
                f = f.replace('\n', '').strip("/").replace("/", "\\")
            else:
                f = f.replace('\n', '').strip("/")
            if os.path.exists(os.path.join(usb_mount, "ldlinux.sys")):
                try:
                    os.chmod(os.path.join(usb_mount, "ldlinux.sys"), 0o777)
                    os.unlink(os.path.join(usb_mount, "ldlinux.sys"))
                except:
                    gen.log('Could not remove ldlinux.sys')

            if os.path.exists(os.path.join(usb_mount, f)):

                if os.path.isdir(os.path.join(usb_mount, f)):
                    gen.log("Removing directory " + (os.path.join(usb_mount, f)))
                    shutil.rmtree(os.path.join(usb_mount, f))

                elif os.path.isfile(os.path.join(usb_mount, f)):
                    gen.log("Removing file " + (os.path.join(usb_mount, f)))
                    os.remove(os.path.join(usb_mount, f))

        if os.path.exists(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "generic.cfg")):
            with open(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "generic.cfg"), "r") as generic_cfg:
                if platform.system() == "Windows":
                    generic = generic_cfg.read().replace('\n', '').replace("/", "\\")
                else:
                    generic = generic_cfg.read().replace('\n', '')
                if os.path.exists(os.path.join(usb_mount, generic.strip("/"))):
                    os.remove(os.path.join(usb_mount, generic.strip("/")))
    if platform.system() == 'Linux':
        gen.log('Removed files from ' + config.uninstall_distro_dir_name)
        gen.log('Syncing....')
        os.sync()




def uninstall_distro():
    """
    Uninstall selected distro from selected USB disk.
    :param config.usb_disk: Path of the USB disk
    :param config.uninstall_distro_dir_name: Directory where the distro is installed
    :param _distro: Generic name applied to distro install by multibootusb
    :return:
    """
    usb_details = details(config.usb_disk)
    usb_mount = usb_details['mount_point']

    if platform.system() == 'Linux':
        os.sync()
        # remove 'immutable' from files on ext2/3/4 fs
        if usb_mount:
            subprocess.call("chattr -i -R %s/* 2>/dev/null" % usb_mount, shell=True)

    if os.path.exists(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "iso_file_list.cfg")):
        with open(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "iso_file_list.cfg"), "r") as f:
            config.iso_file_list = f.readlines()

    for path, subdirs, files in os.walk(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name)):
        for name in files:
            if name.endswith('ldlinux.sys') or name.endswith('ldlinux.c32'):
                os.chmod(os.path.join(path, name), 0o777)
                os.unlink(os.path.join(path, name))

    if config.distro == "opensuse":
        if os.path.exists(os.path.join(usb_mount, config.uninstall_distro_dir_name + ".iso")):
            os.remove(os.path.join(usb_mount, config.uninstall_distro_dir_name + ".iso"))
    elif config.distro == "windows" or config.distro == "alpine" or config.distro == "generic":
        delete_frm_file_list()

    if config.distro == "ipfire":
        files = os.listdir(usb_mount)
        for f in files:
            if f.endswith('.tlz'):
                os.remove(os.path.join(usb_mount, f))
        if os.path.exists(os.path.join(usb_mount, "distro.img")):
            os.remove(os.path.join(usb_mount, "distro.img"))
    elif config.distro == "trinity-rescue":
        shutil.rmtree(os.path.join(usb_mount, "trk3"))

    if os.path.exists(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name)):
        if platform.system() == 'Linux':
            os.sync()
        shutil.rmtree(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name))

    delete_frm_file_list()

    update_sys_cfg_file()
    update_grub_cfg_file()

    # Check if bootx64.efi is replaced by distro
    efi_grub_img = os.path.join(config.usb_mount, 'EFI', 'BOOT', 'bootx64.efi')
    if not os.path.exists(efi_grub_img):
        gen.log('EFI image does not exist. Copying now...')
        os.makedirs(os.path.join(config.usb_mount, 'EFI', 'BOOT'), exist_ok=True)
        shutil.copy2(gen.resource_path(os.path.join("data", "EFI", "BOOT", "bootx64.efi")),
                     os.path.join(config.usb_mount, 'EFI', 'BOOT'))
    elif not gen.grub_efi_exist(efi_grub_img):
        gen.log('EFI image overwritten by distro install. Replacing it now...')
        shutil.copy2(gen.resource_path(os.path.join("data", "EFI", "BOOT", "bootx64.efi")),
                     os.path.join(config.usb_mount, 'EFI', 'BOOT'))
    else:
        gen.log('multibootusb EFI image already exist. Not copying...')


def update_sys_cfg_file():
    """
    Main function to remove uninstall distro specific operations.
    :return:
    """
    if platform.system() == 'Linux':
        os.sync()

    sys_cfg_file = os.path.join(config.usb_mount, "multibootusb", "syslinux.cfg")
    if not os.path.exists(sys_cfg_file):
        gen.log("syslinux.cfg file not found for updating changes.")
    else:
        gen.log("Updating syslinux.cfg file...")
        string = open(sys_cfg_file).read()
        string = re.sub(r'#start ' + config.uninstall_distro_dir_name + '.*?' + '#end ' + config.uninstall_distro_dir_name + '\s*', '', string, flags=re.DOTALL)
        config_file = open(sys_cfg_file, "w")
        config_file.write(string)
        config_file.close()


def update_grub_cfg_file():
    """
    Main function to remove uninstall distro name from the grub.cfg file.
    :return:
    """
    if platform.system() == 'Linux':
        os.sync()

    grub_cfg_file = os.path.join(config.usb_mount, "multibootusb", "grub", "grub.cfg")
    if not os.path.exists(grub_cfg_file):
        gen.log("grub.cfg file not found for updating changes.")
    else:
        gen.log("Updating grub.cfg file...")
        string = open(grub_cfg_file).read()
        string = re.sub(r'#start ' + config.uninstall_distro_dir_name + '.*?' + '#end ' + config.uninstall_distro_dir_name + '\s*', '', string, flags=re.DOTALL)
        config_file = open(grub_cfg_file, "w")
        config_file.write(string)
        config_file.close()


def uninstall_progress():
    """
    Calculate uninstall progress percentage.
    :return:
    """
    from . import progressbar
    usb_details = details(config.usb_disk)
    usb_mount = usb_details['mount_point']
    if platform.system() == 'Linux':
        os.sync()

    if os.path.exists(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "multibootusb.cfg")):
        with open(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name, "multibootusb.cfg"),
                  "r") as multibootusb_cfg:
            config.distro = multibootusb_cfg.read().replace('\n', '')
    else:
        config.distro = ""
    gen.log("Installed distro type is " +  config.distro)

    if config.distro == "opensuse":
        if os.path.exists(os.path.join(usb_mount, config.uninstall_distro_dir_name) + ".iso"):
            folder_size_to_remove = os.path.getsize(os.path.join(usb_mount, config.uninstall_distro_dir_name) + ".iso")
        else:
            folder_size_to_remove = 0
        folder_size_to_remove += disk_usage(str(usb_mount) + "/multibootusb/" + config.uninstall_distro_dir_name).used
    elif config.distro == "windows" or config.distro == "Windows":
        if os.path.exists(os.path.join(usb_mount, "SOURCES")):
            folder_size_to_remove = disk_usage(str(usb_mount) + "/SOURCES").used
        else:
            folder_size_to_remove = disk_usage(str(usb_mount) + "/SSTR").used
    elif config.distro == "ipfire":
        folder_size_to_remove = disk_usage(str(usb_mount) + "/multibootusb/" + config.uninstall_distro_dir_name).used
        files = os.listdir(os.path.join(str(usb_mount)))
        for f in files:
            if f.endswith('.tlz'):
                folder_size_to_remove += os.path.getsize(os.path.join(config.usb_mount, f))
    elif config.distro == "trinity-rescue":
        folder_size_to_remove = disk_usage(os.path.join(usb_mount, "trk3")).used
        folder_size_to_remove += disk_usage(usb_mount + "/multibootusb/" + config.uninstall_distro_dir_name).used
    else:

        folder_size_to_remove = disk_usage(os.path.join(usb_mount, "multibootusb", config.uninstall_distro_dir_name)).used

    thrd = threading.Thread(target=unin_distro, name="uninstall_progress")
    initial_usb_size = disk_usage(usb_mount).used
    thrd.start()
    config.status_text = "Uninstalling " + config.uninstall_distro_dir_name
    pbar = progressbar.ProgressBar(maxval=100).start()  # bar = progressbar.ProgressBar(redirect_stdout=True)
    while thrd.is_alive():
        current_size = disk_usage(usb_mount).used
        diff_size = int(initial_usb_size - current_size)
        config.percentage = round(float(diff_size) / folder_size_to_remove * 100)
        if config.percentage > 100:
            config.percentage = 100

        pbar.update(config.percentage)

    if not thrd.is_alive():
        config.persistence = 0
        config.status_text = ""



# -*- coding: utf-8 -*-
# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


"""
    pyudev._util
    ============

    Internal utilities

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""


from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

try:
    from subprocess import check_output
except ImportError:
    from pyudev._compat import check_output

import os
import sys
import stat
import errno

import six


def ensure_byte_string(value):
    """
    Return the given ``value`` as bytestring.

    If the given ``value`` is not a byte string, but a real unicode string, it
    is encoded with the filesystem encoding (as in
    :func:`sys.getfilesystemencoding()`).
    """
    if not isinstance(value, bytes):
        value = value.encode(sys.getfilesystemencoding())
    return value


def ensure_unicode_string(value):
    """
    Return the given ``value`` as unicode string.

    If the given ``value`` is not a unicode string, but a byte string, it is
    decoded with the filesystem encoding (as in
    :func:`sys.getfilesystemencoding()`).
    """
    if not isinstance(value, six.text_type):
        value = value.decode(sys.getfilesystemencoding())
    return value


def property_value_to_bytes(value):
    """
    Return a byte string, which represents the given ``value`` in a way
    suitable as raw value of an udev property.

    If ``value`` is a boolean object, it is converted to ``'1'`` or ``'0'``,
    depending on whether ``value`` is ``True`` or ``False``.  If ``value`` is a
    byte string already, it is returned unchanged.  Anything else is simply
    converted to a unicode string, and then passed to
    :func:`ensure_byte_string`.
    """
    # udev represents boolean values as 1 or 0, therefore an explicit
    # conversion to int is required for boolean values
    if isinstance(value, bool):
        value = int(value)
    if isinstance(value, bytes):
        return value
    else:
        return ensure_byte_string(six.text_type(value))


def string_to_bool(value):
    """
    Convert the given unicode string ``value`` to a boolean object.

    If ``value`` is ``'1'``, ``True`` is returned.  If ``value`` is ``'0'``,
    ``False`` is returned.  Any other value raises a
    :exc:`~exceptions.ValueError`.
    """
    if value not in ('1', '0'):
        raise ValueError('Not a boolean value: {0!r}'.format(value))
    return value == '1'


def udev_list_iterate(libudev, entry):
    """
    Iteration helper for udev list entry objects.

    Yield a tuple ``(name, value)``.  ``name`` and ``value`` are bytestrings
    containing the name and the value of the list entry.  The exact contents
    depend on the list iterated over.
    """
    while entry:
        name = libudev.udev_list_entry_get_name(entry)
        value = libudev.udev_list_entry_get_value(entry)
        yield (name, value)
        entry = libudev.udev_list_entry_get_next(entry)


def get_device_type(filename):
    """
    Get the device type of a device file.

    ``filename`` is a string containing the path of a device file.

    Return ``'char'`` if ``filename`` is a character device, or ``'block'`` if
    ``filename`` is a block device.  Raise :exc:`~exceptions.ValueError` if
    ``filename`` is no device file at all.  Raise
    :exc:`~exceptions.EnvironmentError` if ``filename`` does not exist or if
    its metadata was inaccessible.

    .. versionadded:: 0.15
    """
    mode = os.stat(filename).st_mode
    if stat.S_ISCHR(mode):
        return 'char'
    elif stat.S_ISBLK(mode):
        return 'block'
    else:
        raise ValueError('not a device file: {0!r}'.format(filename))


def eintr_retry_call(func, *args, **kwargs):
    """
    Handle interruptions to an interruptible system call.

    Run an interruptible system call in a loop and retry if it raises EINTR.
    The signal calls that may raise EINTR prior to Python 3.5 are listed in
    PEP 0475.  Any calls to these functions must be wrapped in eintr_retry_call
    in order to handle EINTR returns in older versions of Python.

    This function is safe to use under Python 3.5 and newer since the wrapped
    function will simply return without raising EINTR.

    This function is based on _eintr_retry_call in python's subprocess.py.
    """

    # select.error inherits from Exception instead of OSError in Python 2
    import select

    while True:
        try:
            return func(*args, **kwargs)
        except (OSError, IOError, select.error) as err:
            # If this is not an IOError or OSError, it's the old select.error
            # type, which means that the errno is only accessible via subscript
            if isinstance(err, (OSError, IOError)):
                error_code = err.errno
            else:
                error_code = err.args[0]

            if error_code == errno.EINTR:
                continue
            raise

def udev_version():
    """
    Get the version of the underlying udev library.

    udev doesn't use a standard major-minor versioning scheme, but instead
    labels releases with a single consecutive number.  Consequently, the
    version number returned by this function is a single integer, and not a
    tuple (like for instance the interpreter version in
    :data:`sys.version_info`).

    As libudev itself does not provide a function to query the version number,
    this function calls the ``udevadm`` utility, so be prepared to catch
    :exc:`~exceptions.EnvironmentError` and
    :exc:`~subprocess.CalledProcessError` if you call this function.

    Return the version number as single integer.  Raise
    :exc:`~exceptions.ValueError`, if the version number retrieved from udev
    could not be converted to an integer.  Raise
    :exc:`~exceptions.EnvironmentError`, if ``udevadm`` was not found, or could
    not be executed.  Raise :exc:`subprocess.CalledProcessError`, if
    ``udevadm`` returned a non-zero exit code.  On Python 2.7 or newer, the
    ``output`` attribute of this exception is correctly set.

    .. versionadded:: 0.8
    """
    output = ensure_unicode_string(check_output(['udevadm', '--version']))
    return int(output.strip())



# -*- coding: utf-8 -*-
# Copyright (C) 2015 mulhern <amulhern@redhat.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""
    pyudev.device
    =============

    Device class implementation of :mod:`pyudev`.

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""


from ._device import Attributes
from ._device import Device
from ._device import Devices
from ._device import Tags



# Author(s) information
__authors__     = ['Sundar']
__author__      = ','.join(__authors__)
__credits__     = ['Ian Bruce', 'Lee']
__copyright__   = 'Copyright (c) 2014'
__license__     = 'GPL'

# Maintanence information
__maintainer__  = 'Sundar'
__email__       = 'feedback.multibootusb@gmail.com'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Name:     7zip.py
# Purpose:  Wrapper module to list and extract ISO files using 7zip
# Authors:  Sundar
# Licence:  This file is a part of multibootusb package. You can redistribute it or modify
# under the terms of GNU General Public License, v.2 or above

import os
import platform
import subprocess
from . import config
from . import gen

if platform.system() == 'Windows':
    _7zip = gen.quote(gen.resource_path(os.path.join('data', 'tools', '7zip', '7z.exe')))
else:
    _7zip = '7z'


def extract_iso(src, dst, pattern=None, suppress_out=True):
    """
    Simple wrapper function to extract ISO file to destination
    :param src: Path to ISO file
    :param dst: Path to directory where the files are to be extracted
    :param patter: The pattern to match the files to be extracted
    :return:
    """
    # 7z x -y -oC:\path_to_directory X:\path_to_iso_file.iso
    # 7z e archive.zip -oC:\path_to_directory *.cfg *.bin -r
    if platform.system() == 'Windows':
        cli_option = ' -ssc- -bb1'  # Linux does not accept this option (may be due to version diff).
        if suppress_out != '':
            # suppress_out = ' 2> nul'
            suppress_out = ''
    else:
        cli_option = ' -ssc- '
        if suppress_out != '':
            suppress_out = ' 2> /dev/null'

    if not os.path.exists(src):
        gen.log('ISO file could not be found on the location specified.')
        return False
    if not os.path.exists(dst):
        os.makedirs(dst, exist_ok=True)

    if pattern is None:
        _cmd = _7zip + cli_option + ' x -y -o' + gen.quote(dst) + ' ' + gen.quote(src) + suppress_out
    else:
        _cmd = _7zip + cli_option + ' x -y ' + gen.quote(src) + ' -o' + dst + ' ' + gen.quote(pattern) + ' -r' + suppress_out
    gen.log('Executing ==> ' + _cmd)

    config.status_text = 'Status: Extracting ' + os.path.basename(src).strip()
    with open(os.devnull, 'w') as devnull:
        subprocess.call(_cmd, stdin=devnull, stdout=devnull, stderr=devnull, shell=True)


def list_iso(iso_link, suppress_out=True):
    """
    List the content of ISO files. It does'nt work with non 'utf' characters (simply ignores them).
    :param iso_link:Path to ISO link
    :param suppress_out: Option to suppress output to stdout. Default True.
    :return: Path to files and directories as a list
    """
    if platform.system() == 'Windows':
        if suppress_out is True:
            suppress_out = ' 2> nul'
    else:
        if suppress_out is True:
            suppress_out = ' 2> /dev/null'
    if not os.path.exists(iso_link):
        gen.log('Path to ISO link does not exist.')
        return False
    else:
        file_list = []
        _cmd = _7zip + ' l ' + gen.quote(iso_link) + suppress_out
        try:
            _cmd_out = subprocess.check_output(_cmd, stderr=subprocess.PIPE, stdin=subprocess.DEVNULL,
                                               shell=True).decode('utf-8', 'ignore').splitlines()
        except Exception as e:
            gen.log(e)
            _cmd_out = ''
        for line in _cmd_out:
            if '...' in line:
                line = line.split()
                _path = line[-1]
                file_list.append(_path)
        return file_list


def test_iso(iso_link, suppress_out=True):
    """
    Function to test if ISO file is corrupted. Relying only on 7zip.
    :param iso_link: Path to ISO file
    :return: True if test is positive
    """
    # 7z t /path/to/iso/file.iso
    # return value : 0 No error
    # return value : 1 Warning (Non fatal error(s))
    # return value : 2 Fatal error
    # return value : 7 Command line error
    # return value : 8 Not enough memory for operation
    # return value : 255 User stopped the process

    if platform.system() == 'Windows':
        if suppress_out is True:
            suppress_out = ' > nul'
    else:
        if suppress_out is True:
            suppress_out = ' > /dev/null'

    _cmd = _7zip + ' t ' + iso_link + suppress_out

    gen.log('Executing ==> ' + _cmd)

    rc = subprocess.call(_cmd, shell=True)

    return bool(rc in [0, 1])


if __name__ == '__main__':
    # slitaz-4.0.iso
    # ubuntu-16.04-desktop-amd64.iso
    # avg_arl_cdi_all_120_160420a12074.iso
    # haiku-nightly.iso
    # Hiren_BootCD.iso
    file_list = list_iso('../../ubuntu_14_04_backup/Downloads/clonezilla-live-2.4.2-32-amd64.iso')
    for f in file_list:
        print(f)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'about.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_About(object):
    def setupUi(self, About):
        About.setObjectName("About")
        About.resize(498, 369)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(About.sizePolicy().hasHeightForWidth())
        About.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(About)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_11 = QtWidgets.QGridLayout()
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.label_6 = QtWidgets.QLabel(About)
        self.label_6.setObjectName("label_6")
        self.gridLayout_11.addWidget(self.label_6, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_11.addItem(spacerItem, 1, 2, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_11.addItem(spacerItem1, 1, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_11.addItem(spacerItem2, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_11.addItem(spacerItem3, 2, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_11)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.button_close = QtWidgets.QPushButton(About)
        self.button_close.setObjectName("button_close")
        self.horizontalLayout.addWidget(self.button_close)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(About)
        QtCore.QMetaObject.connectSlotsByName(About)

    def retranslateUi(self, About):
        _translate = QtCore.QCoreApplication.translate
        About.setWindowTitle(_translate("About", "Dialog"))
        self.label_6.setText(_translate("About", "<html><head/><body><p align=\"center\">An advanced bootable usb creator with option to install/uninstall multiple distros.</p><p align=\"center\">This software is written in Python and PyQt. </p><p align=\"center\">Copyright  2010-2017  Sundar</p><p align=\"center\"><span style=\" font-weight:600;\">Author(s)</span>: Sundar, Ian Bruce, LiQiong Lee and Alin Trăistaru (alindt)</p><p align=\"center\"><span style=\" font-weight:600;\">Licence</span>: GPL version 2 or later</p><p align=\"center\"><span style=\" font-weight:600;\">Home page</span>: <a href=\"http://multibootusb.org\"><span style=\" text-decoration: underline; color:#0000ff;\">http://multibootusb.org</span></a></p><p align=\"center\"><span style=\" font-weight:600;\">Help/Email</span>: feedback.multibootusb@gmail.com</p><p align=\"center\"><span style=\" font-weight:600;\">Source Code</span>: <a href=\"https://github.com/mbusb/multibootusb\"><span style=\" text-decoration: underline; color:#0000ff;\">https://github.com/mbusb/multibootusb</span></a></p><p><br/></p></body></html>"))
        self.button_close.setText(_translate("About", "Close"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    About = QtWidgets.QDialog()
    ui = Ui_About()
    ui.setupUi(About)
    About.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


"""
    pyudev.core
    ===========

    Core types and functions of :mod:`pyudev`.

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""


from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from .device import Devices
from ._errors import DeviceNotFoundAtPathError
from ._ctypeslib.libudev import ERROR_CHECKERS
from ._ctypeslib.libudev import SIGNATURES
from ._ctypeslib.utils import load_ctypes_library

from ._util import ensure_byte_string
from ._util import ensure_unicode_string
from ._util import property_value_to_bytes
from ._util import udev_list_iterate


class Context(object):
    """
    A device database connection.

    This class represents a connection to the udev device database, and is
    really *the* central object to access udev.  You need an instance of this
    class for almost anything else in pyudev.

    This class itself gives access to various udev configuration data (e.g.
    :attr:`sys_path`, :attr:`device_path`), and provides device enumeration
    (:meth:`list_devices()`).

    Instances of this class can directly be given as ``udev *`` to functions
    wrapped through :mod:`ctypes`.
    """

    def __init__(self):
        """
        Create a new context.
        """
        self._libudev = load_ctypes_library('udev', SIGNATURES, ERROR_CHECKERS)
        self._as_parameter_ = self._libudev.udev_new()

    def __del__(self):
        self._libudev.udev_unref(self)

    @property
    def sys_path(self):
        """
        The ``sysfs`` mount point defaulting to ``/sys'`` as unicode string.
        """
        if hasattr(self._libudev, 'udev_get_sys_path'):
            return ensure_unicode_string(self._libudev.udev_get_sys_path(self))
        else:
            # Fixed path since udev 183
            return '/sys'

    @property
    def device_path(self):
        """
        The device directory path defaulting to ``/dev`` as unicode string.
        """
        if hasattr(self._libudev, 'udev_get_dev_path'):
            return ensure_unicode_string(self._libudev.udev_get_dev_path(self))
        else:
            # Fixed path since udev 183
            return '/dev'

    @property
    def run_path(self):
        """
        The run runtime directory path defaulting to ``/run`` as unicode
        string.

        .. udevversion:: 167

        .. versionadded:: 0.10
        """
        if hasattr(self._libudev, 'udev_get_run_path'):
            return ensure_unicode_string(self._libudev.udev_get_run_path(self))
        else:
            return '/run/udev'

    @property
    def log_priority(self):
        """
        The logging priority of the interal logging facitility of udev as
        integer with a standard :mod:`syslog` priority.  Assign to this
        property to change the logging priority.

        UDev uses the standard :mod:`syslog` priorities.  Constants for these
        priorities are defined in the :mod:`syslog` module in the standard
        library:

        >>> import syslog
        >>> context = pyudev.Context()
        >>> context.log_priority = syslog.LOG_DEBUG

        .. versionadded:: 0.9
        """
        return self._libudev.udev_get_log_priority(self)

    @log_priority.setter
    def log_priority(self, value):
        """
        Set the log priority.

        :param int value: the log priority.
        """
        self._libudev.udev_set_log_priority(self, value)

    def list_devices(self, **kwargs):
        """
        List all available devices.

        The arguments of this method are the same as for
        :meth:`Enumerator.match()`.  In fact, the arguments are simply passed
        straight to method :meth:`~Enumerator.match()`.

        This function creates and returns an :class:`Enumerator` object,
        that can be used to filter the list of devices, and eventually
        retrieve :class:`Device` objects representing matching devices.

        .. versionchanged:: 0.8
           Accept keyword arguments now for easy matching.
        """
        return Enumerator(self).match(**kwargs)


class Enumerator(object):
    """
    A filtered iterable of devices.

    To retrieve devices, simply iterate over an instance of this class.
    This operation yields :class:`Device` objects representing the available
    devices.

    Before iteration the device list can be filtered by subsystem or by
    property values using :meth:`match_subsystem` and
    :meth:`match_property`.  Multiple subsystem (property) filters are
    combined using a logical OR, filters of different types are combined
    using a logical AND.  The following filter for instance::

        devices.match_subsystem('block').match_property(
            'ID_TYPE', 'disk').match_property('DEVTYPE', 'disk')

    means the following::

        subsystem == 'block' and (ID_TYPE == 'disk' or DEVTYPE == 'disk')

    Once added, a filter cannot be removed anymore.  Create a new object
    instead.

    Instances of this class can directly be given as given ``udev_enumerate *``
    to functions wrapped through :mod:`ctypes`.
    """

    def __init__(self, context):
        """
        Create a new enumerator with the given ``context`` (a
        :class:`Context` instance).

        While you can create objects of this class directly, this is not
        recommended.  Call :method:`Context.list_devices()` instead.
        """
        if not isinstance(context, Context):
            raise TypeError('Invalid context object')
        self.context = context
        self._as_parameter_ = context._libudev.udev_enumerate_new(context)
        self._libudev = context._libudev

    def __del__(self):
        self._libudev.udev_enumerate_unref(self)

    def match(self, **kwargs):
        """
        Include devices according to the rules defined by the keyword
        arguments.  These keyword arguments are interpreted as follows:

        - The value for the keyword argument ``subsystem`` is forwarded to
          :meth:`match_subsystem()`.
        - The value for the keyword argument ``sys_name`` is forwared to
          :meth:`match_sys_name()`.
        - The value for the keyword argument ``tag`` is forwared to
          :meth:`match_tag()`.
        - The value for the keyword argument ``parent`` is forwared to
          :meth:`match_parent()`.
        - All other keyword arguments are forwareded one by one to
          :meth:`match_property()`.  The keyword argument itself is interpreted
          as property name, the value of the keyword argument as the property
          value.

        All keyword arguments are optional, calling this method without no
        arguments at all is simply a noop.

        Return the instance again.

        .. versionadded:: 0.8

        .. versionchanged:: 0.13
           Add ``parent`` keyword.
        """
        subsystem = kwargs.pop('subsystem', None)
        if subsystem is not None:
            self.match_subsystem(subsystem)
        sys_name = kwargs.pop('sys_name', None)
        if sys_name is not None:
            self.match_sys_name(sys_name)
        tag = kwargs.pop('tag', None)
        if tag is not None:
            self.match_tag(tag)
        parent = kwargs.pop('parent', None)
        if parent is not None:
            self.match_parent(parent)
        for prop, value in kwargs.items():
            self.match_property(prop, value)
        return self

    def match_subsystem(self, subsystem, nomatch=False):
        """
        Include all devices, which are part of the given ``subsystem``.

        ``subsystem`` is either a unicode string or a byte string, containing
        the name of the subsystem.  If ``nomatch`` is ``True`` (default is
        ``False``), the match is inverted:  A device is only included if it is
        *not* part of the given ``subsystem``.

        Note that, if a device has no subsystem, it is not included either
        with value of ``nomatch`` True or with value of ``nomatch`` False.

        Return the instance again.
        """
        match = self._libudev.udev_enumerate_add_nomatch_subsystem \
           if nomatch else self._libudev.udev_enumerate_add_match_subsystem
        match(self, ensure_byte_string(subsystem))
        return self

    def match_sys_name(self, sys_name):
        """
        Include all devices with the given name.

        ``sys_name`` is a byte or unicode string containing the device name.

        Return the instance again.

        .. versionadded:: 0.8
        """
        self._libudev.udev_enumerate_add_match_sysname(
            self, ensure_byte_string(sys_name))
        return self

    def match_property(self, prop, value):
        """
        Include all devices, whose ``prop`` has the given ``value``.

        ``prop`` is either a unicode string or a byte string, containing
        the name of the property to match.  ``value`` is a property value,
        being one of the following types:

        - :func:`int`
        - :func:`bool`
        - A byte string
        - Anything convertable to a unicode string (including a unicode string
          itself)

        Return the instance again.
        """
        self._libudev.udev_enumerate_add_match_property(
            self, ensure_byte_string(prop), property_value_to_bytes(value))
        return self

    def match_attribute(self, attribute, value, nomatch=False):
        """
        Include all devices, whose ``attribute`` has the given ``value``.

        ``attribute`` is either a unicode string or a byte string, containing
        the name of a sys attribute to match.  ``value`` is an attribute value,
        being one of the following types:

        - :func:`int`,
        - :func:`bool`
        - A byte string
        - Anything convertable to a unicode string (including a unicode string
          itself)

        If ``nomatch`` is ``True`` (default is ``False``), the match is
        inverted:  A device is include if the ``attribute`` does *not* match
        the given ``value``.

        .. note::

           If ``nomatch`` is ``True``, devices which do not have the given
           ``attribute`` at all are also included.  In other words, with
           ``nomatch=True`` the given ``attribute`` is *not* guaranteed to
           exist on all returned devices.

        Return the instance again.
        """
        match = (self._libudev.udev_enumerate_add_match_sysattr
                 if not nomatch else
                 self._libudev.udev_enumerate_add_nomatch_sysattr)
        match(self, ensure_byte_string(attribute),
              property_value_to_bytes(value))
        return self

    def match_tag(self, tag):
        """
        Include all devices, which have the given ``tag`` attached.

        ``tag`` is a byte or unicode string containing the tag name.

        Return the instance again.

        .. udevversion:: 154

        .. versionadded:: 0.6
        """
        self._libudev.udev_enumerate_add_match_tag(self, ensure_byte_string(tag))
        return self

    def match_is_initialized(self):
        """
        Include only devices, which are initialized.

        Initialized devices have properly set device node permissions and
        context, and are (in case of network devices) fully renamed.

        Currently this will not affect devices which do not have device nodes
        and are not network interfaces.

        Return the instance again.

        .. seealso:: :attr:`Device.is_initialized`

        .. udevversion:: 165

        .. versionadded:: 0.8
        """
        self._libudev.udev_enumerate_add_match_is_initialized(self)
        return self

    def match_parent(self, parent):
        """
        Include all devices on the subtree of the given ``parent`` device.

        The ``parent`` device itself is also included.

        ``parent`` is a :class:`~pyudev.Device`.

        Return the instance again.

        .. udevversion:: 172

        .. versionadded:: 0.13
        """
        self._libudev.udev_enumerate_add_match_parent(self, parent)
        return self

    def __iter__(self):
        """
        Iterate over all matching devices.

        Yield :class:`Device` objects.
        """
        self._libudev.udev_enumerate_scan_devices(self)
        entry = self._libudev.udev_enumerate_get_list_entry(self)
        for name, _ in udev_list_iterate(self._libudev, entry):
            try:
                yield Devices.from_sys_path(self.context, name)
            except DeviceNotFoundAtPathError:
                continue



# -*- coding: utf-8 -*-
# Copyright (C) 2013 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""
    pyudev._ctypeslib._errorcheckers
    ================================

    Error checkers for ctypes wrappers.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


import os
import errno
from ctypes import get_errno


ERRNO_EXCEPTIONS = {
    errno.ENOMEM: MemoryError,
    errno.EOVERFLOW: OverflowError,
    errno.EINVAL: ValueError
}


def exception_from_errno(errnum):
    """Create an exception from ``errnum``.

    ``errnum`` is an integral error number.

    Return an exception object appropriate to ``errnum``.

    """
    exception = ERRNO_EXCEPTIONS.get(errnum)
    errorstr = os.strerror(errnum)
    if exception is not None:
        return exception(errorstr)
    else:
        return EnvironmentError(errnum, errorstr)


def check_negative_errorcode(result, func, *args):
    """Error checker for funtions, which return negative error codes.

    If ``result`` is smaller than ``0``, it is interpreted as negative error
    code, and an appropriate exception is raised:

    - ``-ENOMEM`` raises a :exc:`~exceptions.MemoryError`
    - ``-EOVERFLOW`` raises a :exc:`~exceptions.OverflowError`
    - all other error codes raise :exc:`~exceptions.EnvironmentError`

    If result is greater or equal to ``0``, it is returned unchanged.

    """
    if result < 0:
        # udev returns the *negative* errno code at this point
        errnum = -result
        raise exception_from_errno(errnum)
    else:
        return result


def check_errno_on_nonzero_return(result, func, *args):
    """Error checker to check the system ``errno`` as returned by
    :func:`ctypes.get_errno()`.

    If ``result`` is not ``0``, an exception according to this errno is raised.
    Otherwise nothing happens.

    """
    if result != 0:
        errnum = get_errno()
        if errnum != 0:
            raise exception_from_errno(errnum)
    return result


def check_errno_on_null_pointer_return(result, func, *args):
    """Error checker to check the system ``errno`` as returned by
    :func:`ctypes.get_errno()`.

    If ``result`` is a null pointer, an exception according to this errno is
    raised.  Otherwise nothing happens.

    """
    # pylint: disable=invalid-name
    if not result:
        errnum = get_errno()
        if errnum != 0:
            raise exception_from_errno(errnum)
    return result



# -*- coding: utf-8 -*-
# Copyright (C) 2010, 2011, 2012, 2013 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


"""
    pyudev._qt_base
    ===============

    Base mixin class for Qt4,Qt5 support.

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import six

from pyudev.device import Device

class MonitorObserverMixin(object):
    """
    Base mixin for pyqt monitor observers.
    """
    # pylint: disable=too-few-public-methods

    def _setup_notifier(self, monitor, notifier_class):
        self.monitor = monitor
        self.notifier = notifier_class(
            monitor.fileno(), notifier_class.Read, self)
        self.notifier.activated[int].connect(self._process_udev_event)

    @property
    def enabled(self):
        """
        Whether this observer is enabled or not.

        If ``True`` (the default), this observer is enabled, and emits events.
        Otherwise it is disabled and does not emit any events.  This merely
        reflects the state of the ``enabled`` property of the underlying
        :attr:`notifier`.

        .. versionadded:: 0.14
        """
        return self.notifier.isEnabled()

    @enabled.setter
    def enabled(self, value):
        self.notifier.setEnabled(value)

    def _process_udev_event(self):
        """
        Attempt to receive a single device event from the monitor, process
        the event and emit corresponding signals.

        Called by ``QSocketNotifier``, if data is available on the udev
        monitoring socket.
        """
        device = self.monitor.poll(timeout=0)
        if device is not None:
            self._emit_event(device)

    def _emit_event(self, device):
        self.deviceEvent.emit(device)


class QUDevMonitorObserverMixin(MonitorObserverMixin):
    """
    Obsolete monitor observer mixin.
    """
    # pylint: disable=too-few-public-methods

    def _setup_notifier(self, monitor, notifier_class):
        MonitorObserverMixin._setup_notifier(self, monitor, notifier_class)
        self._action_signal_map = {
            'add': self.deviceAdded, 'remove': self.deviceRemoved,
            'change': self.deviceChanged, 'move': self.deviceMoved,
        }
        import warnings
        warnings.warn('Will be removed in 1.0. '
                      'Use pyudev.pyqt4.MonitorObserver instead.',
                      DeprecationWarning)

    def _emit_event(self, device):
        self.deviceEvent.emit(device.action, device)
        signal = self._action_signal_map.get(device.action)
        if signal is not None:
            signal.emit(device)

def make_init(qobject, socket_notifier):
    """
    Generates an initializer to observer the given ``monitor``
    (a :class:`~pyudev.Monitor`):

    ``parent`` is the parent :class:`~PyQt{4,5}.QtCore.QObject` of this
    object.  It is passed unchanged to the inherited constructor of
    :class:`~PyQt{4,5}.QtCore.QObject`.
    """

    def __init__(self, monitor, parent=None):
        qobject.__init__(self, parent)
        # pylint: disable=protected-access
        self._setup_notifier(monitor, socket_notifier)

    return __init__


class MonitorObserverGenerator(object):
    """
    Class to generate a MonitorObserver class.
    """
    # pylint: disable=too-few-public-methods

    @staticmethod
    def make_monitor_observer(qobject, signal, socket_notifier):
        """Generates an observer for device events integrating into the
        PyQt{4,5} mainloop.

        This class inherits :class:`~PyQt{4,5}.QtCore.QObject` to turn device
        events into Qt signals:

        >>> from pyudev import Context, Monitor
        >>> from pyudev.pyqt4 import MonitorObserver
        >>> context = Context()
        >>> monitor = Monitor.from_netlink(context)
        >>> monitor.filter_by(subsystem='input')
        >>> observer = MonitorObserver(monitor)
        >>> def device_event(device):
        ...     print('event {0} on device {1}'.format(device.action, device))
        >>> observer.deviceEvent.connect(device_event)
        >>> monitor.start()

        This class is a child of :class:`~{PySide, PyQt{4,5}}.QtCore.QObject`.

        """
        return type(
           str("MonitorObserver"),
           (qobject, MonitorObserverMixin),
           {
              str("__init__") : make_init(qobject, socket_notifier),
              str("deviceEvent") : signal(Device)
           }
        )



class QUDevMonitorObserverGenerator(object):
    """
    Class to generate a MonitorObserver class.
    """
    # pylint: disable=too-few-public-methods

    @staticmethod
    def make_monitor_observer(qobject, signal, socket_notifier):
        """Generates an observer for device events integrating into the
        PyQt{4,5} mainloop.

        This class inherits :class:`~PyQt{4,5}.QtCore.QObject` to turn device
        events into Qt signals:

        >>> from pyudev import Context, Monitor
        >>> from pyudev.pyqt4 import MonitorObserver
        >>> context = Context()
        >>> monitor = Monitor.from_netlink(context)
        >>> monitor.filter_by(subsystem='input')
        >>> observer = MonitorObserver(monitor)
        >>> def device_event(device):
        ...     print('event {0} on device {1}'.format(device.action, device))
        >>> observer.deviceEvent.connect(device_event)
        >>> monitor.start()

        This class is a child of :class:`~{PyQt{4,5}, PySide}.QtCore.QObject`.

        """
        return type(
           str("QUDevMonitorObserver"),
           (qobject, QUDevMonitorObserverMixin),
           {
              str("__init__") : make_init(qobject, socket_notifier),
              #: emitted upon arbitrary device events
              str("deviceEvent") : signal(six.text_type, Device),
              #: emitted if a device was added
              str("deviceAdded") : signal(Device),
              #: emitted if a device was removed
              str("deviceRemoved") : signal(Device),
              #: emitted if a device was changed
              str("deviceChanged") : signal(Device),
              #: emitted if a device was moved
              str("deviceMoved") : signal(Device)
           }
        )



import os
import inspect
from . import config

try:
    from colorama import init as colorama_init, Fore, Back, Style
except:
    try:
        from .colorama import init as colorama_init, Fore, Back, Style
    except Exception as e:
        raise e
finally:
    try:
        colorama_init(autoreset=True)
        class colors:
            HEADER = Fore.MAGENTA + Style.BRIGHT
            OKBLUE = Fore.BLUE + Style.BRIGHT
            OKGREEN = Fore.GREEN + Style.BRIGHT
            WARNING = Fore.YELLOW + Style.BRIGHT
            FAIL = Fore.RED + Style.BRIGHT
            RESET = Style.RESET_ALL
            BOLD = Style.BRIGHT
            UNDERLINE = '\033[4m'
    except NameError:
        class colors:
            HEADER = ''
            OKBLUE = ''
            OKGREEN = ''
            WARNING = ''
            FAIL = ''
            RESET = ''
            BOLD = ''
            UNDERLINE = ''

def fnCall():
    """
    Prints filename:line:function for parent and grandparent.
    """

    if not config.debug:
        return

    print(colors.OKGREEN + colors.BOLD + "=== DEBUG === | %s:%d:%s() called from %s:%d:%s()" % (
        inspect.stack()[1][1].split(os.sep)[-1],
        inspect.stack()[1][2],
        inspect.stack()[1][3],
        inspect.stack()[2][1].split(os.sep)[-1],
        inspect.stack()[2][2],
        inspect.stack()[2][3],
        )+colors.RESET)



# -*- coding: utf-8 -*-
# Copyright (C) 2011 Sebastian Wiesner <lunaryorn@gmail.com>

# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.

# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


"""
    pyudev._compat
    ==============

    Compatibility for Python versions, that lack certain functions.

    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@gmail.com>
"""


from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from subprocess import Popen, CalledProcessError, PIPE


def check_output(command):
    """
    Compatibility with :func:`subprocess.check_output` from Python 2.7 and
    upwards.
    """
    proc = Popen(command, stdout=PIPE)
    output = proc.communicate()[0]
    if proc.returncode != 0:
        raise CalledProcessError(proc.returncode, command)
    return output
